#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import ABCMeta, abstractmethod, abstractproperty


class AbstractLocale(object):
    """ An abstract base class defining the required api for a locale 
    object to be used by the framework.

    """
    __metaclass__ = ABCMeta

    #--------------------------------------------------------------------------
    # Properties
    #--------------------------------------------------------------------------
    @abstractproperty
    def name(self):
        """ The RFC 1766 language code for the locale.

        """
        raise NotImplementedError

    @abstractproperty
    def encoding(self):
        """ The encoding for strings in the locale.

        """
        raise NotImplementedError
        
    @abstractproperty
    def am_string(self):
        """ The string for "AM" time in the locale.

        """
        raise NotImplementedError
    
    @abstractproperty
    def pm_string(self):
        """ The string for "PM" time in the locale.

        """
        raise NotImplementedError

    @abstractproperty
    def decimal_point(self):
        """ The decimal separator for the locale.

        """
        raise NotImplementedError
    
    @abstractproperty
    def monetary_decimal_point(self):
        """ The decimal separator for monetary values for the locale.

        """
        raise NotImplementedError

    @abstractproperty
    def exponential(self):
        """ The exponential character for the locale.

        """
        raise NotImplementedError

    @abstractproperty
    def thousands_separator(self):
        """ The thousands separator for the locale.

        """
        raise NotImplementedError
    
    @abstractproperty
    def monetary_thousands_separator(self):
        """ The thousands separator for monetary values for the locale.

        """
        raise NotImplementedError

    @abstractproperty
    def negative_sign(self):
        """ The monetary negative sign for the locale.

        """
        raise NotImplementedError
        
    @abstractproperty
    def positive_sign(self):
        """ The monetary positive sign for the locale.

        """
        raise NotImplementedError
    
    #--------------------------------------------------------------------------
    # Methods
    #--------------------------------------------------------------------------
    @abstractmethod
    def month_name(self, month_int, abbreviated=False):
        """ The month name for the given month integer.

        Parameters
        ----------
        month_int : int
            An integer in the range 1 to 12 inclusive.
        
        abbreviated : bool, optional
            If True, return the abbreviate version of the month name.
            The default is False.
        
        Returns
        -------
        result : unicode string
            The month name or an empty string in the month int is out
            of range.

        """
        raise NotImplementedError
    
    @abstractmethod
    def day_name(self, day_int, abbreviated=False):
        """ The day name for the given day integer.

        Parameters
        ----------
        day_int : int
            An integer in the range 1 to 7 inclusive.
        
        abbreviated : bool, optional
            If True, return the abbreviate version of the day name.
            The default is False.
        
        Returns
        -------
        result : unicode string
            The day name or an empty string in the day int is out
            of range.

        """
        raise NotImplementedError
        
    @abstractmethod
    def time_format(self, abbreviated=False):
        """ The format string for a time value.

        Parameters
        ----------
        abbreviated : bool, optional
            If True, return the abbreviate version of the time format.
            The defaults is False.

        Returns
        -------
        result : unicode string
            The time format string.

        """
        raise NotImplementedError

    @abstractmethod
    def date_format(self, abbreviated=False):
        """ The format string for a date value.

        Parameters
        ----------
        abbreviated : bool, optional
            If True, return the abbreviated version of the date format.
            The defaults is False.

        Returns
        -------
        result : unicode string
            The date format string.

        """
        raise NotImplementedError
    
    @abstractmethod
    def datetime_format(self, abbreviated=False):
        """ The format string for a datetime value.

        Parameters
        ----------
        abbreviated : bool, optional
            If True, return the abbreviated version of the datetime
            format. The defaults is False.

        Returns
        -------
        result : unicode string
            The datetime format string.

        """
        raise NotImplementedError
    
    #--------------------------------------------------------------------------
    # Conversions
    #--------------------------------------------------------------------------
    @abstractmethod
    def to_time(self, time_str, abbreviated=False):
        """ Parse the given time string into a datetime.time object.

        Parameters
        ----------
        time_str : string
            A string formatted according to time_format().

        abbreviated : bool, optional
            Whether or not the time string is using the abbreviated 
            format. The default is False.
        
        Returns
        -------
        result : datetime.time
            The datetime.time object for the string.

        """
        raise NotImplementedError

    @abstractmethod
    def to_date(self, date_str, abbreviated=False):
        """ Parse the date string into a datetime.date object.

        Parameters
        ----------
        date_str : string
            A string formatted according to date_format().

        abbreviated : bool, optional
            Whether or not the date string is using the abbreviated 
            format. The default is False.
        
        Returns
        -------
        result : datetime.date
            The datetime.date object for the string.

        """
        raise NotImplementedError
    
    @abstractmethod
    def to_datetime(self, datetime_str, abbreviated=False):
        """ Parse the datetime string into a datetime.datetime object.

        Parameters
        ----------
        datetime_str : string
            A string formatted according to datetime_format().

        abbreviated : bool, optional
            Whether or not the datetime string is using the abbreviated 
            format. The default is False.
        
        Returns
        -------
        result : datetime.datetime
            The datetime.datetime object for the string.

        """
        raise NotImplementedError

    @abstractmethod
    def to_int(self, int_str, base=10):
        """ Parse the string into an integer using the given base.

        Parameters
        ----------
        int_str : string
            The string representing the integer.
        
        base : int, optional
            The number base to use when parsing the string. The valid
            values are the same as those passed to Python's builtin 
            'int'. The default is 10.
        
        Returns
        -------
        result : int
            The parsed integer.

        """ 
        raise NotImplementedError
        
    @abstractmethod
    def to_long(self, long_str, base=10):
        """ Parse the string into a long integer using the given base.

        Parameters
        ----------
        long_str : string
            The string representing the integer.
        
        base : int, optional
            The number base to use when parsing the string. The valid
            values are the same as those passed to Python's builtin 
            'int'. The default is 10.
        
        Returns
        -------
        result : long
            The parsed long integer.

        """ 
        raise NotImplementedError

    @abstractmethod
    def to_float(self, float_str):
        """ Parse the given string into a float.

        Parameters
        ----------
        float_str : string
            The string representing the float.
        
        Returns
        -------
        result : float
            The parsed float.

        """
        raise NotImplementedError

    @abstractmethod
    def to_complex(self, complex_str):
        """ Parse the given string into a complex.

        Parameters
        ----------
        complex_str : string
            The string representing the complex.
        
        Returns
        -------
        result : complex
            The parsed complex.

        """
        raise NotImplementedError

    @abstractmethod
    def to_currency(self, currency_str, symbol=True, group=False, intl=False):
        """ Parse the given currency string into a float.

        Parameters
        ----------
        currency_str : string
            The string representing the currency.
        
        symbol : bool, optional
            If True, the currency symbol is included in the string. The
            default is True.
        
        group : bool, optional
            If True, the grouping marks are included in the string. The
            default is False.
        
        intl : bool, optional
            If True, the international version of the currency symbol
            is used in the string. This is only used if 'symbol' is
            True. The default is False.

        Returns
        -------
        result : float
            The parsed float.

        """
        raise NotImplementedError

    @abstractmethod
    def from_time(self, time_obj, abbreviated=False):
        """ Convert the given time object into a locale specific string.

        Parameters
        ----------
        time_obj : datetime.time
            The time object for conversion.
        
        abbreviated : bool, optional
            Whether or not the time string should use the abbreviated 
            format. The default is False.

        Returns
        -------
        result : unicode string
            The locale formatted time string.

        """
        raise NotImplementedError
    
    @abstractmethod
    def from_date(self, date_obj, abbreviated=False):
        """ Convert the given date object into a locale specific string.

        Parameters
        ----------
        date_obj : datetime.date
            The date object for conversion.
        
        abbreviated : bool, optional
            Whether or not the date string should use the abbreviated 
            format. The default is False.

        Returns
        -------
        result : unicode string
            The locale formatted date string.

        """
        raise NotImplementedError
    
    @abstractmethod
    def from_datetime(self, datetime_obj, abbreviated=False):
        """ Convert the given datetime object into a locale specific 
        string.

        Parameters
        ----------
        datetime_obj : datetime.datetime
            The datetime object for conversion.
        
        abbreviated : bool, optional
            Whether or not the datetime string should use the abbreviated 
            format. The default is False.

        Returns
        -------
        result : unicode string
            The locale formatted datetime string.

        """
        raise NotImplementedError
    
    @abstractmethod
    def from_int(self, value, base=10, group=False):
        """ Format the integer value according to the locale.

        Parameters
        ----------
        value : int
            The integer to convert a string.
        
        base : int, optional
            The number base to use when parsing the string. The valid
            values are the same as those passed to Python's builtin 
            'int'. The default is 10.

        group : bool, optional
            Whether or not to add group separators in the result. The
            default is False.
        
        Returns
        -------
        result : unicode string
            The locale formatted integer string.
        
        """
        raise NotImplementedError

    @abstractmethod
    def from_long(self, value, base=10, group=False):
        """ Format the long integer value according to the locale.

        Parameters
        ----------
        value : long
            The long integer to convert a string.
        
        base : int, optional
            The number base to use when parsing the string. The valid
            values are the same as those passed to Python's builtin 
            'int'. The default is 10.

        group : bool, optional
            Whether or not to add group separators in the result. The
            default is False.
        
        Returns
        -------
        result : unicode string
            The locale formatted long integer string.
        
        """
        raise NotImplementedError

    @abstractmethod
    def from_float(self, value, group=False, prec=-1):
        """ Format the float value according to the locale.

        Parameters
        ----------
        value : float
            The float to convert a string.
        
        group : bool, optional
            Whether or not to add group separators in the result. The
            default is False.
        
        prec : int, optional
            The number of signficant digits to display in the string.
            The default is -1 which means the decision is left up
            to the implementor.

        Returns
        -------
        result : unicode string
            The locale formatted float string.
        
        """
        raise NotImplementedError

    @abstractmethod
    def from_complex(self, value, group=False, prec=-1):
        """ Format the complex value according to the locale.

        Parameters
        ----------
        value : complex
            The complex to convert a string.
        
        group : bool, optional
            Whether or not to add group separators in the result. The
            default is False.
        
        prec : int, optional
            The number of signficant digits to display for each part.
            The default is -1 which means the decision is left up
            to the implementor.

        Returns
        -------
        result : unicode string
            The locale formatted complex string.
        
        """
        raise NotImplementedError

    @abstractmethod
    def from_currency(self, value, symbol=True, group=False, intl=False):
        """ Convert the given value into a locale specific currency
        string.

        Parameters
        ----------
        value : int, long, or float
            The value representing the currency
        
        symbol : bool, optional
            If True, the currency symbol is included in the string. The
            default is True.
        
        group : bool, optional
            If True, the grouping marks are included in the string. The
            default is False.
        
        intl : bool, optional
            If True, the international version of the currency symbol
            is used in the string. This is only used if 'symbol' is
            True. The default is False.
            
        Returns
        -------
        result : unicode string
            The locale formatted currency string.

        """
        raise NotImplementedError




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    Property, Instance, Either, Enum, Tuple, Int, cached_property,
    on_trait_change,
)

from .constraints_widget import (
    ConstraintsWidget, AbstractTkConstraintsWidget,
)
from .container import Container
from .layout_task_handler import LayoutTaskHandler
from .widget_component import WidgetComponent

from ..layout.geometry import Size


#: Enum trait describing the scrollbar policies that can be assigned to 
#: the horizontal and vertical scrollbars.
ScrollbarPolicy = Enum('as_needed', 'always_on', 'always_off')


class AbstractTkScrollArea(AbstractTkConstraintsWidget):
    """ The abstract toolkit ScrollArea interface. A toolkit scroll area 
    is responsible for handling changes on a shell ScrollArea and proxying 
    those changes to and from its internal toolkit widget.

    """
    @abstractmethod
    def shell_horizontal_scrollbar_policy_changed(self, policy):
        raise NotImplementedError

    @abstractmethod
    def shell_vertical_scrollbar_policy_changed(self, policy):
        raise NotImplementedError

    @abstractmethod
    def shell_scrolled_component_changed(self, component):
        raise NotImplementedError

    @abstractmethod
    def horizontal_scrollbar_thickness(self):
        raise NotImplementedError

    @abstractmethod
    def vertical_scrollbar_thickness(self):
        raise NotImplementedError

    @abstractmethod
    def scroll_to_position(self, position, margin):
        raise NotImplementedError

class ScrollArea(LayoutTaskHandler, ConstraintsWidget):
    """ A LayoutComponent subclass that displays just a single child in
    a scrollable area.

    """
    #: A read-only property which returns the scrolling component
    #: for the area, or None if one is not defined.
    scrolled_component = Property(
        Instance(WidgetComponent), depends_on='children',
    )

    #: The horizontal scroll policy.
    horizontal_scrollbar_policy = ScrollbarPolicy

    #: The vertical scroll policy.
    vertical_scrollbar_policy = ScrollbarPolicy

    #: The preferred (width, height) size of the scroll area. Each item
    #: of the tuple can be either None or an integer. If None, then that 
    #: component is requested from the child widget's size hint. As a
    #: default, the height is fixed to 200 and the width is taken from the
    #: widget. This accounts for the typical use case of display a lot
    #: of vertically laid-out information in a confined area.
    preferred_size = Tuple(Either(None, Int, default=None), 
                           Either(Int, None, default=200))

    #: How strongly a component hugs it's contents' width. Scroll
    #: areas do not hug their width and are free to expand.
    hug_width = 'ignore'

    #: How strongly a component hugs it's contents' height. Scroll
    #: areas do not hug their height and are free to expand.
    hug_height = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkScrollArea)

    def scroll_to_position(self, position, margin):
        """ Scrolls the area such that position is visible with a
        minimum of margin_size points surrounding position.

        Parameters
        ----------
        position : (x, y)
            The x,y tuple of the position to be made visible.

        margin : (x-margin, y-margin)
            The x,y size tuple of the margins to keep around position.
        
        """
        self.abstract_obj.scroll_to_position(position, margin)

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_scrolled_component(self):
        """ The property getter for the 'scrolled_component' attribute.

        """
        children = self.children
        n = len(children)
        if n == 0:
            res = None
        elif n == 1:
            res = children[0]
        else:
            msg = ('A ScrollArea can have at most 1 widget child. Got '
                   '%s instead.')
            raise ValueError(msg % n)
        return res
        
    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    @on_trait_change('scrolled_component, preferred_size')
    def _on_scroll_area_deps_changed(self):
        """ A change handler which requests a relayout when the scrolled
        component or the preferred size changes, provided that the 
        component is initialized.

        """
        if self.initialized:
            self.request_relayout()

    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def _setup_finalize(self):
        """ Overridden setup method to set the min size of the component
        once its layout has been initialized.

        """
        super(ScrollArea, self)._setup_finalize()
        self.update_min_scrolled_size()

    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def size_hint(self):
        """ A reimplemented parent class method which uses the given 
        preferred size when computing the size hint, but falls back
        to the scrolled component's size hint if a preferred size is 
        not given.

        """
        # TODO - We probably want to honor the hug and clip properties
        # on the scrolled component when computing its size hint. This
        # will allow any Container in which we may be embedded to 
        # compute an appropriate size hint from ours.
        width, height = self.preferred_size
        width_hint, height_hint = self.scrolled_component.size_hint()
        
        if width is None:
            r_width = width_hint
            if height is not None and height <= height_hint:
                r_width += self.abstract_obj.vertical_scrollbar_thickness()
        else:
            r_width = width
        
        if height is None:
            r_height = height_hint
            if width is not None and width <= width_hint:
                r_height += self.abstract_obj.horizontal_scrollbar_thickness()
        else:
            r_height = height
        
        return Size(r_width, r_height)

    def do_relayout(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        perform necessary update activity when a relayout it requested.

        """
        # This method is called whenever a relayout is requested.
        # We update the size of the scrolled components and fire 
        # off a size hint updated event so that the parent can
        # peform a relayout.
        self.update_min_scrolled_size()
        self.size_hint_updated()

    def update_min_scrolled_size(self):
        """ Updates the minimum size of the scrolled component with its
        computed minimum size, or its size hint.

        """
        scrolled = self.scrolled_component
        if scrolled is not None:
            if isinstance(scrolled, Container):
                min_size = scrolled.compute_min_size()
            else:
                min_size = scrolled.size_hint()
            scrolled.set_min_size(min_size)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import Instance

from .toggle_control import ToggleControl, AbstractTkToggleControl


class AbstractTkRadioButton(AbstractTkToggleControl):
    pass


class RadioButton(ToggleControl):
    """ A radio button widget derived from IToggleElement 
    
    Use a radio button to toggle the value of a boolean field. For a 
    group of radio buttons with the same widget parent, only one radio 
    button may be selected at a time. This makes groups of radio buttons 
    useful for selecting amongst a discrete set of values. For multiple 
    groups of independent radio buttons, place them in their own Container.

    See Also
    --------
    ToggleControl
    
    """
    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkRadioButton)




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from weakref import ref

from ..qt.QtCore import Qt

from ....noncomponents.abstract_dock_manager import AbstractTkDockManager


#: A mapping from Enaml dock area values to Qt dock area enum values.
DOCK_AREA_MAP = {
    'left': Qt.LeftDockWidgetArea,
    'right': Qt.RightDockWidgetArea,
    'top': Qt.TopDockWidgetArea,
    'bottom': Qt.BottomDockWidgetArea,
    'all': Qt.AllDockWidgetAreas,
}


class QtDockManager(AbstractTkDockManager):
    """ 

    """
    def __init__(self, *dock_panes):
        """ Initialize a DockManager instance.

        Parameters
        ----------
        *dock_panes
            The initial DockPane instances to use with the manager.

        """
        self._panes = list(dock_panes)
        self._window_ref = lambda: None

    def _get_main_window(self):
        """ The property getter for the 'main_window' property.

        """
        return self._window_ref()
    
    def _set_main_window(self, window):
        """ The property setter for the 'main_window' property.

        """
        old_main = self._window_ref()
        self._window_ref = ref(window)
        if old_main is not None:
            old_main.dock_manager = None
        curr_panes = self._panes
        self._panes = []
        for pane in curr_panes:
            self.add_pane(pane)

    main_window = property(_get_main_window, _set_main_window)
                
    def add_pane(self, pane):
        """ Add a dock pane to the manager and hence the window.

        Parameters
        ----------
        pane : DockPane
            The DockPane instance to add to the manager. If the pane is
            already being managed, this method call is a no-op.
        
        """
        if pane not in self._panes:
            self._panes.append(pane)
            window = self._window_ref()
            if window is not None:
                qwindow = window.toolkit_widget
                if qwindow is not None:
                    if not pane.initialized:
                        pane.setup(qwindow)
                    qarea = DOCK_AREA_MAP[pane.dock_area]
                    qwindow.addDockWidget(qarea, pane.toolkit_widget)

    def remove_pane(self, pane):
        """ Remove a dock pane from the manager and hence the window.

        Parameters
        ----------
        pane : DockPane
            The DockPane instance to remove from the manager. If the
            pane is not being managed, this method call is a no-op.
        
        """
        if pane in self._panes:
            self._panes.remove(pane)
            window = self._window_ref()
            if window is not None:
                qwindow = window.toolkit_widget
                if qwindow is not None:
                    qwindow.removeDockWidget(pane.toolkit_widget)
    
    def panes(self):
        """ Get the list of dock panes currently being managed.

        Returns
        -------
        result : list
            The list of DockPane instance being managed by this manager.
        
        """
        return self._panes[:]




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .wx_test_assistant import WXTestAssistant, skip_nonwindows
from .. import radio_button


@skip_nonwindows
class TestWxRadioButton(WXTestAssistant, radio_button.TestRadioButton):
    """ WXRadioButton tests. """

    def get_value(self, button):
        """ Get the checked state of a radio button.

        """
        self.process_wx_events(self.app)
        return button.GetValue()

    def get_text(self, button):
        """ Get the label of a button.

        """
        return button.GetLabel()



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" A simple publisher api that acts as a throttling broadcaster for a 
high frequency data source.

"""
import threading
import time
from abc import ABCMeta, abstractmethod
import numpy as np
from enaml.core.signaling import Signal


TIMESTAMP_FLOAT32_DTYPE = np.dtype(
    [('index', np.float64), ('value', np.float32)]
)


class AbstractPublisher(object):
    """ An abstract base class representing a data republisher. A 
    publisher subcsribes to a given feed id but throttles updates 
    to a rate which is manageable for the given task.

    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def start(self):
        """ Start the publisher feed.

        """
        raise NotImplementedError

    @abstractmethod
    def stop(self):
        """ Stop the publisher feed.

        """
        raise NotImplementedError

    @abstractmethod
    def subscribe(self, callback):
        """ Subscribe a callback to the feed which will be called with
        a structured numpy array when data is available.

        """
        raise NotImplementedError

    @abstractmethod
    def unsubscribe(self, callback):
        """ Unsubscribe a callback previously registered via subscribe.

        """
        raise NotImplementedError


class BasePublisher(AbstractPublisher):
    """ A base publisher class which provides a simple event loop for
    processing a feed.

    """
    #: The signal used to implement notification
    _data_changed = Signal()

    #: The default event loop frequency
    frequency = 30

    def start_event_loop(self):
        """ A simple event loop implemented with a daemon thread.

        """
        def inner(publisher):
            while publisher._go_thread:
                buf = publisher.process_feed()
                publisher._data_changed(buf)
                time.sleep(1.0 / publisher.frequency)
        self._go_thread = True
        thread = threading.Thread(target=inner, args=(self,))
        thread.daemon = True
        thread.start()
    
    def stop_event_loop(self):
        """ Stop the event loop thread.

        """
        self._go_thread = False
    
    def subscribe(self, callback):
        """ Subscribe a callback to the publisher. Callbacks will be
        notified from a thread.

        """
        self._data_changed.connect(callback)

    def unsubscribe(self, callback):
        """ Unsubscribe a previously registered callback.

        """
        self._data_changed.disconnect(callback)

    @abstractmethod
    def process_feed(self):
        """ Do any necessary processing on the queued feed and return
        the value to be emitted to any listeners. This method will be 
        called from a thread.

        """
        raise NotImplementedError


class NumpyPublisher(BasePublisher):
    """ A concrete implementation of BasePublisher which acummulates the
    data into a numpy array.

    """

    def __init__(self, data_gen, id, size=1000000, frequency=30, sampler=None):
        """ Initialize a NumpyPublisher.

        Parameters
        ----------
        data_gen : AbstractDataGenerator
            An AbstractDataGenerator instance which provides the feeds
            we are interested in.
        
        id : string
            The identifier for the particular data feed from the data
            generator which should be republished to any subscribers.
        
        size : int, optional
            The size of the buffer to queue incoming data. The default
            is 1e6.
        
        frequency : int, optional
            The frequency with which to republish data. The default
            is 30 (Hz)
        
        sampler : AbstractDataSampler, optional
            If provided, an instance of AbstractDataSampler which has
            the opportunity to subsample the data before it is 
            republished to subcribers.
        
        """
        self.data_gen = data_gen
        self.feed_id = id
        self.frequency = frequency
        self.sampler = sampler
        self._buffer = np.empty(size, dtype=TIMESTAMP_FLOAT32_DTYPE)
        self.size = size
        self.index = 0
        self.lock = threading.Lock()
    
    #--------------------------------------------------------------------------
    # AbstractPublisher Implementation
    #--------------------------------------------------------------------------
    def start(self):
        """ Bind to the data feed with the given id and start the 
        event loop.

        """
        self.data_gen.subscribe(self.feed_id, self.insert)
        self.start_event_loop()

    def stop(self):
        """ Stop the publisher feed.

        """
        self.stop_event_loop()

    def process_feed(self):
        """ Process and return the current buffered data.

        """
        with self.lock:
            data = np.copy(self._buffer[:self.index])
        sampler = self.sampler
        if sampler is not None:
            data = sampler.sample(data)
        self.index = 0
        return data

    #--------------------------------------------------------------------------
    # Processing Helpers
    #--------------------------------------------------------------------------    
    def insert(self, value):
        """ The callback from the data feed which inserts the data point
        into the internal buffer.

        """
        with self.lock:
            if self.index >= self.size:
                raise BufferError
            id, idx, val = value
            item = self._buffer[self.index]
            item['index'] = idx
            item['value'] = val
            self.index += 1
            



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_base_widget_component import QtBaseWidgetComponent

from ...components.action import AbstractTkAction


class QtAction(QtBaseWidgetComponent, AbstractTkAction):
    """ A Qt4 implementation of Action.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QAction object.

        """
        self.widget = QtGui.QAction(parent)

    def initialize(self):
        """ Initializes the underlying QAction object.

        """
        super(QtAction, self).initialize()
        shell = self.shell_obj
        self.set_enabled(shell.enabled)
        self.set_text(shell.text)
        self.set_checkable(shell.checkable)
        self.set_checked(shell.checked)
        self.set_status_tip(shell.status_tip)
        self.set_tool_tip(shell.tool_tip)
        self.set_icon(shell.icon)
        self.set_description(shell.description)
        self.set_separator(shell.separator)
        
    def bind(self):
        """ Binds the event handlers for the underlying QAction object.

        """
        super(QtAction, self).bind()
        widget = self.widget
        widget.triggered.connect(self.on_triggered)
        widget.hovered.connect(self.on_hovered)
        widget.toggled.connect(self.on_toggled)

    #--------------------------------------------------------------------------
    # Change Handlers 
    #--------------------------------------------------------------------------
    def shell_enabled_changed(self, enabled):
        """ The change handler for the 'enabled' attribute of the shell
        object. Sets the widget enabled according to the given boolean.

        """
        self.set_enabled(enabled)

    def shell_text_changed(self, text):
        """ The change handler for the 'text' attribute on the shell
        object.

        """
        self.set_text(text)
    
    def shell_checkable_changed(self, checkable):
        """ The change handler for the 'checkable' attribute on the 
        shell object.

        """
        self.set_checkable(checkable)
    
    def shell_checked_changed(self, checked):
        """ The change handler for the 'checked' attribute on the
        shell object.

        """
        self.set_checked(checked)

    def shell_status_tip_changed(self, status_tip):
        """ The change handler for the 'status_tip' attribute on the 
        shell object.

        """
        self.set_status_tip(status_tip)
    
    def shell_tool_tip_changed(self, tool_tip):
        """ The change handler for the 'tool_tip' attribute on the 
        shell object.

        """
        self.set_tool_tip(tool_tip)

    def shell_icon_changed(self, icon):
        """ The change handler for the 'icon' attribute on the shell 
        object.

        """
        self.set_icon(icon)
    
    def shell_description_changed(self, description):
        """ The change handler for the 'description' attribute on the 
        shell object.

        """
        self.set_description(description)

    def shell_separator_changed(self, separator):
        """ The change handler for the 'separator' attribute on the
        shell object.

        """
        self.set_separator(separator)

    #--------------------------------------------------------------------------
    # Signal Handlers
    #--------------------------------------------------------------------------
    def on_triggered(self):
        """ A signal handler for the 'triggered' signal of the QAction.

        """
        self.shell_obj.triggered()

    def on_hovered(self):
        """ A signal handler for the 'hovered' signal of the QAction.

        """
        self.shell_obj.hovered()

    def on_toggled(self):
        """ A signal handler for the 'toggled' signal of the QAction.

        """
        shell = self.shell_obj
        shell.checked = self.widget.isChecked()
        shell.toggled()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_enabled(self, enabled):
        """ Sets the enabled state of the action.

        """
        self.widget.setEnabled(enabled)

    def set_text(self, text):
        """ Sets the text of the action.

        """
        self.widget.setText(text)

    def set_checkable(self, checkable):
        """ Sets whether or not the action is checkable.

        """
        self.widget.setCheckable(checkable)
    
    def set_checked(self, checked):
        """ Sets whether or not the action is checked.

        """
        self.widget.setChecked(checked)

    def set_status_tip(self, status_tip):
        """ Sets the status tip text for the action.

        """
        self.widget.setStatusTip(status_tip)
    
    def set_tool_tip(self, tool_tip):
        """ Sets the tool tip text for the action.

        """
        self.widget.setToolTip(tool_tip)
    
    def set_icon(self, icon):
        """ Sets the icon for the action.

        """
        if icon is None:
            qicon = QtGui.QIcon()
        else:
            qicon = icon.as_QIcon()
        self.widget.setIcon(qicon)
    
    def set_description(self, description):
        """ Sets the description text for the action.

        """
        self.widget.setWhatsThis(description)

    def set_separator(self, separator):
        """ Sets whether or not the QAction is a separator.

        """
        self.widget.setSeparator(separator)




# -*- coding: utf-8 -*-
"""
    sphinx.domains.enaml
    ~~~~~~~~~~~~~~~~~~~~~

    The Enaml domain.
    (Subclassed extensively from the Python domain)

    :copyright: Copyright 2012 by Enthought, Inc
"""

from sphinx.locale import l_, _
from sphinx.domains import Domain, ObjType, Index
from sphinx.util.nodes import make_refnode

# Subclass objects defined in the Python domain
from sphinx.domains.python import PythonDomain, PyModulelevel, PyClasslike, PyClassmember
from sphinx.domains.python import PyModule,  PyXRefRole

## class EnamlDefn(PyClasslike):
##     """
##     Enaml defn formatting particulars
##     """

##     def get_signature_prefix(self, sig):
##         return 'Enaml defn '

##     def get_index_text(self, modname, name_cls):
##         return _('%s (Enaml defn)') % name_cls[0]

class EnamlDeclaration(PyClasslike):
    """
    Enaml Declaration formatting particulars.
    """

    def get_signature_prefix(self, sig):
        ## import pdb; pdb.set_trace()
        return 'Enaml declared component '

    def get_index_text(self, modname, name_cls):
        """return a signature to appear in the index listing"""

        if modname is not None:
            # if the declaration was in a module, report it 
            return _('%s (Enaml declaration in %s)') % (name_cls[0], modname)
        else:
            # give a simple description if the directive is self-contained
            return _('%s (Enaml declaration)') % (name_cls[0])

class EnamlDomain(PythonDomain):
    """Enaml language domain."""
    name = 'enaml'
    label = 'Enaml'
    object_types = {
        'enaml_module':    ObjType(l_('enaml module'),    'mod', 'obj'),
        ## 'enaml_defn':      ObjType(l_('enaml built-in'),  'enaml_comp', 'obj'),
        'enaml_decl':      ObjType(l_('enaml derived'),   'enaml_comp', 'obj'),
        }

    directives = {
        ## 'enaml_defn':      EnamlDefn,
        'enaml_decl':      EnamlDeclaration,
        'enaml_module':    PyModule,
        }
    roles = {
        'mod':   PyXRefRole(),
        'enaml_comp': PyXRefRole(),
        }



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from casuarius import ConstraintVariable


class BoxModel(object):
    """ Provides ConstraintVariables describing a box.

    Primitive Variables:
        - left
        - top
        - width
        - height
    
    Derived Variables:
        - right
        - bottom
        - v_center
        - h_center

    """
    def __init__(self, component):
        label = '{0}_{1:x}'.format(type(component).__name__, id(component))
        for primitive in ('left', 'top', 'width', 'height'):
            var = ConstraintVariable('{0}_{1}'.format(primitive, label))
            setattr(self, primitive, var) 
        self.right = self.left + self.width
        self.bottom = self.top + self.height
        self.v_center = self.top + self.height / 2.0
        self.h_center = self.left + self.width / 2.0        


class PaddingBoxModel(BoxModel):
    """ Provides ConstraintVariables describing a box with padding.

    Primitive Variables:
        - padding_[left|right|top|bottom]
    
    Derived Variables:
        - contents_[left|top|right|bottom|width|height|v_center|h_center]
    
    """
    def __init__(self, component):
        super(PaddingBoxModel, self).__init__(component)
        label = '{0}_{1:x}'.format(type(component).__name__, id(component))
        for primitive in ('left', 'right', 'top', 'bottom'):
           attr = 'padding_{0}'.format(primitive)
           var = ConstraintVariable('{0}_{1}'.format(attr, label))
           setattr(self, attr, var)
        self.contents_left = self.left + self.padding_left
        self.contents_top = self.top + self.padding_top
        self.contents_right = self.right - self.padding_right
        self.contents_bottom = self.bottom - self.padding_bottom
        self.contents_width = self.contents_right - self.contents_left
        self.contents_height = self.contents_bottom - self.contents_top
        self.contents_v_center = self.contents_top + self.contents_height / 2.0
        self.contents_h_center = self.contents_left + self.contents_width / 2.0




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui


def q_color_from_color(color):
    """ Converts an enaml Color into a QtGui.QColor instance.

    """
    if not color:
        res = QtGui.QColor()
    else:
        res = QtGui.QColor(*color)
    return res


font_style_map = {
    'normal': QtGui.QFont.StyleNormal,
    'italic': QtGui.QFont.StyleItalic,
    'oblique': QtGui.QFont.StyleOblique,
}


font_family_hint_map = {
    'any': QtGui.QFont.AnyStyle,
    'sans_serif': QtGui.QFont.SansSerif,
    'helvetica': QtGui.QFont.Helvetica,
    'serif': QtGui.QFont.Serif,
    'times': QtGui.QFont.Times,
    'type_writer': QtGui.QFont.TypeWriter,
    'courier': QtGui.QFont.Courier,
    'old_english': QtGui.QFont.OldEnglish,
    'decorative': QtGui.QFont.Decorative,
    'monospace': QtGui.QFont.Monospace,
    'fantasy': QtGui.QFont.Fantasy,
    'cursive': QtGui.QFont.Cursive,
    'system': QtGui.QFont.System,
}


def q_font_from_font(font):
    """ Converts an enaml Font into a QtGui.QFont instance.

    """
    q_font = QtGui.QFont()
    if font.family:
        q_font.setFamily(font.family)
    if font.point_size != -1:
        q_font.setPointSize(font.point_size)
    if font.weight != -1:
        q_font.setWeight(font.weight)
    if font.style:
        q_font.setStyle(font_style_map[font.style])
    if font.underline:
        q_font.setUnderline(font.underline)
    if font.strikethrough:
        q_font.setStrikeOut(font.strikethrough)
    if font.family_hint:
        q_font.setStyleHint(font_family_hint_map[font.family_hint])
    # XXX more to go, this is enough to test
    return q_font




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance, Bool

from .abstract_item_view import AbstractItemView, AbstractTkItemView


class AbstractTkTreeView(AbstractTkItemView):
    
    @abstractmethod
    def shell_header_visible_changed(self, visible):
        raise NotImplementedError


class TreeView(AbstractItemView):
    """ A view for tree data.
    
    """
    #: Whether or not the tree header is visible. Defaults to True.
    header_visible = Bool(True)

    #: How strongly a component hugs it's contents' height. TreeViews 
    #: ignore the height hug by default, so they expand freely in height.
    hug_height = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkTreeView)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .wx_test_assistant import WXTestAssistant, skip_nonwindows
from .. import window

@skip_nonwindows
class TestWXWindow(WXTestAssistant, window.TestWindow):
    """ WXWindow tests. """

    def get_title(self, widget):
        """ Get a window's title.

        """
        return widget.GetTitle()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import radio_button


class TestQtRadioButton(QtTestAssistant, radio_button.TestRadioButton):
    """ QtRadioButton tests. """

    def get_value(self, button):
        """ Get the checked state of a radio button.

        """
        return button.isChecked()

    def get_text(self, button):
        """ Get the label of a button.

        """
        return button.text()




#------------------------------------------------------------------------------
#  file: refactor_doc.py
#  License: LICENSE.TXT
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from function_doc import FunctionDoc
from class_doc import ClassDoc
from enaml_decl_doc import EnamlDeclDoc

#------------------------------------------------------------------------------
# Extension definition
#------------------------------------------------------------------------------
def refactor_docstring(app, what, name, obj, options, lines):

    verbose = False
    # if 'component.Component' in name:
        # verbose = True

    if ('class' in what):
        ClassDoc(lines, verbose=verbose)
    elif ('function' in what) or ('method' in what):
        FunctionDoc(lines, verbose=verbose)
    elif ('enaml_decl' in what):
        EnamlDeclDoc(lines, verbose=verbose)

def setup(app):
    app.setup_extension('sphinx.ext.autodoc')
    app.connect('autodoc-process-docstring', refactor_docstring)



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    Bool, Instance, Property, Int, TraitError, Either, Range, on_trait_change,
)

from .control import Control, AbstractTkControl

from ..core.trait_types import Bounded, EnamlEvent
from ..enums import Orientation, TickPosition, PolicyEnum


class AbstractTkSlider(AbstractTkControl):

    @abstractmethod
    def shell_value_changed(self, value):
        raise NotImplementedError

    @abstractmethod
    def shell_minimum_changed(self, minimum):
        raise NotImplementedError

    @abstractmethod
    def shell_maximum_changed(self, maximum):
        raise NotImplementedError

    @abstractmethod
    def shell_tracking_changed(self, tracking):
        raise NotImplementedError

    @abstractmethod
    def shell_single_step_changed(self, single_step):
        raise NotImplementedError

    @abstractmethod
    def shell_page_step_changed(self, page_step):
        raise NotImplementedError

    @abstractmethod
    def shell_tick_interval_changed(self, tick_interval):
        raise NotImplementedError

    @abstractmethod
    def shell_tick_position_changed(self, tick_position):
        raise NotImplementedError

    @abstractmethod
    def shell_orientation_changed(self, orientation):
        raise NotImplementedError


#: The maximum slider value
MAX_SLIDER_VALUE = (1 << 16) - 1


class Slider(Control):
    """ A simple slider widget that can be used to select from a 
    range of values.

    """
    #: The minimum value for the index. To avoid issues where
    #: :attr:`minimum` is higher than :attr:`maximum`. The value is
    #: a positive integer capped by the :attr:`maximum`. If the new
    #: value of :attr:`minimum` make the current position invalid then
    #: the current position is set to :attr:minimum. Default value is 0.
    minimum = Property(Int, depends_on ='_minimum')

    #: The internal minimum storage
    _minimum = Int(0)

    #: The maximum value for the index. Checks make sure that
    #: :attr:`maximum` cannot be lower than :attr:`minimum`. If the
    #: new value of :attr:`maximum` make the current position invalid
    #: then the current position is set to :attr:maximum. The max value
    #: is restricted to 65535, while the default is 100.
    maximum = Property(Int, depends_on ='_maximum')

    #: The internal maximum storage
    _maximum = Int(100)

    #: The span of the slider, a read only property that depends on
    #: :attr:`minimum` and :attr:`maximum`. The span value is used
    #: by a number of properties that adapt the slider's appearence.
    span = Property(Int, depends_on=('minimum', 'maximum'))

    #: The position value of the Slider. The bounds are defined by
    #: :attr:minimum: and :attr:maximum:.
    value = Bounded(low='minimum', high='maximum')

    #: The interval to put between tick marks in slider range units.
    #: Default value is `10`.
    tick_interval = Range(low=1, high='span', value=10, exclude_high=True)

    #: Defines the number of steps that the slider will move when the
    #: user presses the arrow keys. Default is 1.
    single_step = Range(low=1, high='span', value=1)

    #: Defines the number of steps that the slider will move when the
    #: user presses the page_up/page_down keys. Default is 10.
    page_step = Range(low=1, high='span', value=10)

    #: A TickPosition enum value indicating how to display the tick
    #: marks. Note that the orientation takes precedence over the tick 
    #: mark position and an incompatible tick position will be adapted 
    #: according to the current orientation. The default tick position
    #: is 'bottom'.
    tick_position = TickPosition('bottom')

    #: The orientation of the slider. The default orientation is
    #: horizontal. When the orientation is flipped the tick positions
    #: (if set) also adapt to reflect the changes  (e.g. the LEFT
    #: becomes TOP when the orientation becomes horizontal).
    orientation = Orientation

    #: If True, the value is updated while sliding. Otherwise, it is
    #: only updated when the slider is released. Defaults to True.
    tracking = Bool(True)

    #: Fired when the slider is pressed.
    pressed = EnamlEvent

    #: Fired when the slider is released.
    released = EnamlEvent

    #: Fired when the user drags the slider handle via the mouse.
    #: The event payload will be the position of the slider. This is
    #: always fired regardless of the value of :attr:`tracking`
    moved = EnamlEvent

    #: A read only property which indicates whether or not the slider
    #: is pressed down.
    down = Property(Bool, depends_on='_down')

    #: The internal down storage
    _down = Bool

    #: Overridden parent class tait
    abstract_obj = Instance(AbstractTkSlider)

    #: Hug width is redefined as a property to be computed based on the 
    #: orientation of the slider unless overridden by the user.
    hug_width = Property(PolicyEnum, depends_on=['_hug_width', 'orientation'])

    #: Hug height is redefined as a property to be computed based on the 
    #: orientation of the slider unless overridden by the user.
    hug_height = Property(PolicyEnum, depends_on=['_hug_height', 'orientation'])

    #: An internal override trait for hug_width
    _hug_width = Either(None, PolicyEnum, default=None)

    #: An internal override trait for hug_height
    _hug_height = Either(None, PolicyEnum, default=None)

    #--------------------------------------------------------------------------
    # Trait defaults
    #--------------------------------------------------------------------------
    def _get_hug_width(self):
        """ The proper getter for 'hug_width'. Returns a computed hug 
        value unless overridden by the user.

        """
        res = self._hug_width
        if res is None:
            if self.orientation == 'horizontal':
                res = 'ignore'
            else:
                res = 'strong'
        return res

    def _get_hug_height(self):
        """ The proper getter for 'hug_width'. Returns a computed hug 
        value unless overridden by the user.

        """
        res = self._hug_height
        if res is None:
            if self.orientation == 'vertical':
                res = 'ignore'
            else:
                res = 'strong'
        return res

    def _set_hug_width(self, value):
        """ The property setter for 'hug_width'. Overrides the computed
        value.

        """
        self._hug_width = value

    def _set_hug_height(self, value):
        """ The property setter for 'hug_height'. Overrides the computed
        value.

        """
        self._hug_height = value

    #--------------------------------------------------------------------------
    # Property methods
    #--------------------------------------------------------------------------
    def _get_down(self):
        """ The property getter for the 'down' attribute.

        """
        return self._down

    def _get_span(self):
        """ The property getter for the 'length' attribute.

        """
        return (self.maximum - self.minimum) + 1

    def _set_minimum(self, value):
        """ The property setter for the 'minimum' attribute. This
        validates that the value is always smaller than :attr:`maximum`.

        """
        if  (value < 0) or (value > self.maximum):
            msg = ('The minimum value of the slider should be a positive '
                   'integer and smaller than the current maximum ({0}), '
                   'but a value of {1} was given')
            msg = msg.format(self.maximum, value)
            raise TraitError(msg)
        else:
            self._minimum = value

    def _set_maximum(self, value):
        """ The property setter for the 'maximum' attribute. This
        validates that the value is always larger than :attr:`minimum`.

        """
        if  (value < self.minimum) or (value > MAX_SLIDER_VALUE):
            msg = ("The maximum value of the slider should be a positive "
                   "integer and larger than the current minimum ({0}), "
                   "but a value of {1} was given")
            msg = msg.format(self.minimum, value)
            raise TraitError(msg)
        else:
            self._maximum = value

    def _get_maximum(self):
        """ The property getter for the slider maximum.

        """
        return self._maximum

    def _get_minimum(self):
        """ The property getter for the slider minimum.

        """
        return self._minimum

    @on_trait_change('minimum, maximum')
    def _adapt_value(self):
        """ Adapt the value to the boundaries

        """
        if self.initialized:
            self.value = min(max(self.value, self.minimum), self.maximum)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import push_button


class TestQtPushButton(QtTestAssistant, push_button.TestPushButton):
    """ QtPushButton tests. """

    def button_pressed(self):
        """ Press the button programmatically.

        """
        self.widget.pressed.emit()

    def button_released(self):
        """ Release the button programmatically.

        """
        self.widget.released.emit()

    def button_clicked(self):
        """ Click the button programmatically.

        PySide expects 0 arguments for this signal, but PyQt expects 1.

        """
        try:
            self.widget.clicked.emit(False)
        except TypeError:
            self.widget.clicked.emit()



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" Toolkit independed abstract testing classes

The tests.widgets package contains a set of abstract classes that simplify
the testing of the Enaml object components for the implemented toolkits.

All Enaml testcases are children of the
:class:`~enaml.tests.widgets.enaml_test_case.EnamlTestCase` class. The
class implements the main functionality to parse and prepare the Enaml
based View object for testing without initiating the event Loop.

For each Enaml object in the standard set of graphical widgets there exist
a corresponding abstract test class that tests the specific Enaml object
for conformance against the expected interface and behaviour. Each of them
require a set of methods (as described in the class docstring) to be
defined before the class can be created.

"""



#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import ABCMeta, abstractmethod

from ..utils import abstractclassmethod


class AbstractTkIcon(object):
    """ An abstract base class which represents a toolkit independent 
    abstraction of an icon.
    
    Concrete implemetations of the AbstractTkIcon class allow the user 
    to specify different images for different states of the icon, so that
    a component can select the image which is most appropriate for 
    display in the widget.
    
    The icon can specify modes and states, which toolkit objects may use
    to select which image to use for display. The available modes are:
        
        'normal'
            A control in its normal mode
        
        'disabled'
            A control which is disabled or greyed-out
    
        'active'
            A control which is currently activiated
    
        'selected'
            A control in a selected state
    
    Not every mode is used by every backend. 

    The state of an icon may be either:
        
        'on'
            The normal or checked form of the icon.
        
        'off'
            The unchecked form of the icon.
    
    """
    __metaclass__ = ABCMeta
    
    @abstractclassmethod
    def from_file(cls, path):
        """ Create a new icon from a file on disk.

        Parameters
        ----------
        path : string
            The path to the image to use for the default mode and state.
        
        Returns
        -------
        result : AbstractTkIcon
            An new icon instance.

        """
        raise NotImplementedError

    @abstractclassmethod
    def from_image(cls, image):
        """ Create a new icon from an instance of AbstractTkImage using
        the default 'normal' and 'on' states.
                
        Parameters
        ----------
        image : AbstractTkImage
            An appropriate instance of AbstractTkImage to use for the
            default mode and state.
        
        Returns
        -------
        result : AbstractTkIcon
            An new icon instance.

        """
        raise NotImplementedError

    @abstractmethod
    def get_image(self, size, mode='normal', state='on'):
        """ Get an appropriate image instance for the requested size, 
        mode and state.
        
        Parameters
        ----------
        size : (width, height)
            The size of the requested image. The returned image may
            be smaller, but will never be larger than this size.
            
        mode : string, optional
            The mode of the requested image. The default is 'normal'.
            
        state : string, optional
            The state of the requested image. The default is 'on'.
        
        Returns
        -------
        result : AbstractTkImage
            An appropriate image instance.

        """
        raise NotImplementedError

    @abstractmethod
    def add_image(self, image, mode='normal', state='on'):
        """ Add an image instance for use by the icon with the given 
        mode and state.
        
        Parameters
        ----------
        image : AbstractTkImage
            An appropriate image instance.
            
        mode : string, optional
            The mode of the image. The default is 'normal'.
            
        state : string, optional
            The state of the image. The default is 'on'.
        
        """
        raise NotImplementedError

    @abstractmethod
    def actual_size(self, size, mode='normal', state='on'):
        """ Returns the actual size for the requested size, mode, and
        state.

        The returned size may be smaller but will never be larger.

        Parameters
        ----------
        size : (width, height)
            The size of the requested image. The returned image may
            be smaller, but will never be larger than this size.
            
        mode : string, optional
            The mode of the requested image. The default is 'normal'.
            
        state : string, optional
            The state of the requested image. The default is 'on'.
        
        Returns
        -------
        result : (width, height)
            The actual size for the requested size. If no suitable
            match can be found, the result will be (-1, -1).

        """
        raise NotImplementedError
    
    @abstractmethod
    def available_sizes(self, mode='normal', state='on'):
        """ Returns the available image sizes for the given mode and 
        state.
        
        Sizes other than these may be requested, but the implemetation 
        may scale down the image on the fly to the requested size.
        
        Parameters
        ----------
        mode : string, optional
            The requested image mode. The default is 'normal'.
            
        state : string, optional
            The requested image state. The default is 'on'.
        
        Returns
        -------
        results : [(width, height), ... ]
            The list of available sizes for the given mode and state.
            
        """
        raise NotImplementedError




""" Defines the SegmentsOverlay class.
"""

from numpy import array

from enable.api import Component, ColorTrait
from traits.api import Instance, List, Tuple, Range

from chaco.api import AbstractOverlay


# XXX -- This code was copied from a more complicated overlay and pared
#        down to its current state.  There may still be some unecessary code.
#
# XXX -- Requires the padding of the component to be 0.

class SegmentsOverlay(AbstractOverlay):
    """
    Draw selectable line segments.
    """
    
    # The component that this tool overlays
    component = Instance(Component)
    
    # XXX Necessary?
    image_shape = Tuple

    # Color of the line segments.
    line_color = ColorTrait('green')
    
    alpha = Range(low=0.0, high=1.0, value=1.0)

    # The list of line segments.  Each segment is a tuple of the
    # form ((x0, y0), (x1, y1))
    # XXX Maybe use a numpy array?
    segments = List
    
    #------------------------------------------------------------------------
    # Object interface
    #------------------------------------------------------------------------
    
    def __init__(self, component=None, **kwtraits):
        if "component" in kwtraits:
            component = kwtraits["component"]
        super(SegmentsOverlay, self).__init__(**kwtraits)
        self.component = component
        return

    #------------------------------------------------------------------------
    # Drawing tool methods
    #------------------------------------------------------------------------

    def reset(self):
        """ Resets the tool in the current plot.
        """
        if self.component is not None:
            c1, c2 = self._data_corners()
            dx = c2[0] - c1[0]
            dy = c2[1] - c1[1]
            pt1 = (c1[0] + 0.05*dx, c1[1] + 0.05*dy)
            pt2 = (c2[0] - 0.05*dx, c2[1] - 0.05*dy)
            self.points = [pt1, pt2]
            self.event_state = "normal"
            self.line.points = list(self.component.map_screen(array(self.points)))
            self.request_redraw()
        return

    def _activate(self):
        """
        Called by a PlotComponent when this becomes the active tool.
        """
        pass

    def _deactivate(self, component=None):
        """
        Called by a PlotComponent when this is no longer the active tool.
        """
        # XXX  Is this the correct thing to do?  Do we even need to implement
        # this method?
        self.visible = False
        self.request_redraw()
        return


    #------------------------------------------------------------------------
    # override AbstractOverlay methods
    #------------------------------------------------------------------------

    def overlay(self, component, gc, view_bounds, mode="normal"):
        """ Draws this component overlaid on another component.
        
        Implements AbstractOverlay.
        """
        with gc:
            gc.clip_to_rect(component.x, component.y,
                            component.width-1, component.height-1)
            self._really_draw(gc)
        return
    
    def request_redraw(self):
        """ Requests that the component redraw itself. 
        
        Overrides Enable Component.
        """
        self.component.invalidate_draw()
        self.component.request_redraw()
        return

    #------------------------------------------------------------------------
    # Private methods
    #------------------------------------------------------------------------

    def _data_corners(self):
        """Get the dataspace points at the corners of the plot."""
        index_range = self.component.index_mapper.range
        value_range = self.component.value_mapper.range
        pts = [(index_range.low, value_range.low),
                (index_range.high, value_range.high)]
        return pts

    def _map_data(self, point):
        """ Maps values from screen space into data space.
        """
        index_mapper = self.component.index_mapper
        value_mapper = self.component.value_mapper
        if self.component.orientation == 'h':
            ndx = index_mapper.map_data(point[0])
            val = value_mapper.map_data(point[1])
        else:
            ndx = index_mapper.map_data(point[1])
            val = value_mapper.map_data(point[0])
        return (ndx, val)

    def _map_screen(self, point):
        """ Maps values from data space into screen space.
        """
        # Temporary hack--this object should know nothing about "canny_plot"!
        x_mapper = self.component.x_mapper
        y_mapper = self.component.y_mapper

        x = x_mapper.map_screen(point[0])
        y = y_mapper.map_screen(point[1])
        return (x, y)

    def _really_draw(self, gc):
        """Render the line segments."""
        w = self.component.width
        h = self.component.height
        padl, padr, padt, padb = self.component.padding
        aw = w - padl - padr
        ah = h - padt - padb
        outerw, outerh = self.component.outer_bounds
        with gc:
            gc.set_stroke_color(self.line_color_[:3] + (self.alpha,))
            gc.set_line_width(2.5)
            for p0, p1 in self.segments:
                # XXX These calculations should be in a mapper or something...
                x0 = padl + p0[0] * aw / float(self.image_size[1])
                y0 = padb + (self.image_size[0] - p0[1]) * ah / float(self.image_size[0])
                x1 = padl + p1[0] * aw / float(self.image_size[1])
                y1 = padb + (self.image_size[0] - p1[1]) * ah / float(self.image_size[0])
                gc.move_to(x0, y0)
                gc.line_to(x1, y1)
                gc.stroke_path()

        return


    #------------------------------------------------------------------------
    # Trait event handlers
    #------------------------------------------------------------------------
    
    def _component_changed(self, old, new):
        if new:
            self.container = new
        return



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from __future__ import absolute_import

import unittest
import cPickle

from ..layout import geometry


class TestPickleGeometry(unittest.TestCase):
    def check_pickling(self, cls, nargs):
        args = (0,) * nargs
        obj = cls(*args)
        pickled = cPickle.dumps(obj, protocol=cPickle.HIGHEST_PROTOCOL)
        unpickled = cPickle.loads(pickled)
        self.assertEqual(obj, unpickled)

    def test_base_rect(self):
        self.check_pickling(geometry.BaseRect, 4)

    def test_rect(self):
        self.check_pickling(geometry.Rect, 4)

    def test_rect_f(self):
        self.check_pickling(geometry.RectF, 4)

    def test_base_box(self):
        self.check_pickling(geometry.BaseBox, 4)

    def test_box(self):
        self.check_pickling(geometry.Box, 4)

    def test_box_f(self):
        self.check_pickling(geometry.BoxF, 4)

    def test_base_pos(self):
        self.check_pickling(geometry.BasePos, 2)

    def test_pos(self):
        self.check_pickling(geometry.Pos, 2)

    def test_pos_f(self):
        self.check_pickling(geometry.PosF, 2)

    def test_base_size(self):
        self.check_pickling(geometry.BaseSize, 2)

    def test_size(self):
        self.check_pickling(geometry.Size, 2)

    def test_size_f(self):
        self.check_pickling(geometry.SizeF, 2)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" Command-line tool to run .enaml files.

"""
import optparse
import os
import sys
import types

from enaml import imports, default_toolkit, wx_toolkit, qt_toolkit
from enaml.core.parser import parse
from enaml.core.enaml_compiler import EnamlCompiler


toolkits = {
    'default': default_toolkit, 'wx': wx_toolkit, 'qt': qt_toolkit,
}


def main():
    usage = 'usage: %prog [options] enaml_file'
    parser = optparse.OptionParser(usage=usage, description=__doc__)
    parser.add_option('-c', '--component', default='Main',
                      help='The component to view')
    parser.add_option('-t', '--toolkit', default='default',
                      choices=['default', 'wx', 'qt'],
                      help='The toolkit backend to use')
    
    options, args = parser.parse_args()

    if len(args) == 0:
        print 'No .enaml file specified'
        sys.exit()
    elif len(args) > 1:
        print 'Too many files specified'
        sys.exit()
    else:
        enaml_file = args[0]

    with open(enaml_file) as f:
        enaml_code = f.read()
    
    # Parse and compile the Enaml source into a code object    
    ast = parse(enaml_code, filename=enaml_file)
    code = EnamlCompiler.compile(ast, enaml_file)

    # Create a proper module in which to execute the compiled code so
    # that exceptions get reported with better meaning
    module = types.ModuleType('__main__')
    module.__file__ = enaml_file
    ns = module.__dict__

    # Put the directory of the Enaml file first in the path so relative imports
    # can work.
    sys.path.insert(0, os.path.abspath(os.path.dirname(enaml_file)))
    with imports():
        exec code in ns

    with toolkits[options.toolkit]():
        requested = options.component
        if requested in ns:
            component = ns[requested]
            window = component()
            window.show()
        elif 'main' in ns:
            ns['main']()
        else:
            msg = "Could not find component '%s'" % options.component
            print msg


if __name__ == '__main__':
    main()




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtCore, QtGui
from .qt_constraints_widget import QtConstraintsWidget

from ...components.splitter import AbstractTkSplitter


_ORIENTATION_MAP = {
    'horizontal': QtCore.Qt.Horizontal,
    'vertical': QtCore.Qt.Vertical,
}


class QtSplitter(QtConstraintsWidget, AbstractTkSplitter):
    """ A Qt implementation of a Splitter.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QSplitter control.

        """
        self.widget = QtGui.QSplitter(parent)

    def initialize(self):
        """ Intializes the widget with the attributes of this instance.

        """
        super(QtSplitter, self).initialize()
        shell = self.shell_obj
        self.set_orientation(shell.orientation)
        self.set_live_drag(shell.live_drag)
        self.update_children()

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_orientation_changed(self, orientation):
        """ Update the orientation of the widget.

        """
        self.set_orientation(orientation)

    def shell_live_drag_changed(self, live_drag):
        """ The change handler for the 'live_drag' attribut of the shell
        object.

        """
        self.set_live_drag(live_drag)

    def shell_splitter_children_changed(self, children):
        """ The change handler for the 'splitter_children' attribute of 
        the shell object.

        """
        self.update_children()
    
    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_orientation(self, orientation):
        """ Update the orientation of the QSplitter.

        """
        q_orientation = _ORIENTATION_MAP[orientation]
        self.widget.setOrientation(q_orientation)

    def set_live_drag(self, live_drag):
        """ Update the dragging mode of the QSplitter.

        """
        self.widget.setOpaqueResize(live_drag)
        
    def update_children(self):
        """ Update the QSplitter's children with the current 
        children.

        """
        # FIXME: there should be a more efficient way to do this, but for 
        # now just remove all present widgets and add the current ones.
        widget = self.widget
        while widget.count():
            # FIXME: there is no explicit API to remove a widget from
            # a QSplitter, so we need to do it implicitly by unparenting.
            i = widget.count() - 1
            child = widget.widget(i)
            child.setParent(None)
        shell = self.shell_obj
        for child in shell.splitter_children:
            widget.addWidget(child.toolkit_widget)

    def set_splitter_sizes(self, sizes):
        """ Set the initial sizes for the children.

        """
        self.widget.setSizes(sizes)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import ABCMeta, abstractmethod
from heapq import heappush, heappop
from itertools import count
from threading import Lock
    

class ScheduledTask(object):
    """ An object representing a task in the scheduler. 

    """
    #: A sentinel object indicating that the result of the task
    #: is undefined or that the task has not yet been executed.
    undefined = object()

    def __init__(self, callback, args, kwargs):
        """ Initialize a SchedulerTask.

        Parameters
        ----------
        callback : callable
            The callable to run when the task is executed.
        
        args : tuple
            The tuple of positional arguments to pass to the callback.

        kwargs : dict
            The dict of keyword arguments to pass to the callback.
        
        """
        self.__callback = callback
        self.__args = args
        self.__kwargs = kwargs
        self.__result = self.undefined
        self.__valid = True
        self.__pending = True

    #--------------------------------------------------------------------------
    # Private API
    #--------------------------------------------------------------------------
    def _execute(self):
        """ Execute the underlying task. This should only been called
        by the scheduler loop.

        """
        try:
            if self.__valid:
                self.__result = self.__callback(*self.__args, **self.__kwargs)
        finally:
            self.__pending = False

    #--------------------------------------------------------------------------
    # Public API 
    #--------------------------------------------------------------------------
    def pending(self):
        """ Returns True if this task is pending execution, False
        otherwise.

        """
        return self.__pending

    def unschedule(self):
        """ Unschedule the task so that it will not be executed. If
        the task has already been executed, this call has no effect.

        """
        self.__valid = False

    def result(self):
        """ Returns the result of the task, or ScheduledTask.undefined
        if the task has not yet been executed, was unscheduled before 
        execution, or raised an exception on execution.

        """
        return self.__result


class AbstractTkApplication(object):
    """ A thread safe abstract base class that represents a simple gui
    toolkit application object. It provides convienent abstraction for
    common operations such as creating the toolkit app, starting the 
    event loop, and calling functions on the main gui thread. It also
    provides a priority scheduler which runs tasks on the main gui 
    thread.

    """
    __metaclass__ = ABCMeta

    def __init__(self):
        """ Initialize an AbstractApplication.

        """
        # The heap of pending tasks awaiting processing
        self.__heap = []

        # A never-ending counter that serves to create tie-breakers
        # for heap priority
        self.__counter = count()

        # A Lock which protects access to the heap
        self.__heap_lock = Lock()

    #--------------------------------------------------------------------------
    # Private API 
    #--------------------------------------------------------------------------
    def __process_task(self, task):
        """ Processes the given task, then dispatches the next task.

        """
        task._execute()
        self.__next_task()
    
    def __next_task(self):
        """ Pulls the next task off the heap and processes it on the 
        main gui thread.

        """
        heap = self.__heap
        with self.__heap_lock:
            if heap:
                priority, count, task = heappop(heap)
                self.call_on_main(self.__process_task, task)

    #--------------------------------------------------------------------------
    # Public API
    #--------------------------------------------------------------------------
    def schedule(self, callback, args=None, kwargs=None, priority=50):
        """ Schedule a callable to be executed on the main gui thread 
        according to its priority. This call is thread-safe.

        Parameters
        ----------
        callback : callable
            The callable object to be executed. It must be hashable.

        args : tuple, optional
            The args to pass to the callable.

        kwargs : dict, optional
            The keyword arguments to pass to the callable.

        priority : int, optional
            The priority with which to schedule the callable. The 
            'lowest' priority is 100 and indicates the callable will 
            be placed at the end of the queue. The 'highest' priority
            is 0 and indicates that the callable will jump to the 
            front of the queue. The default is 50.

        Returns
        -------
        result : ScheduledTask
            A task object which can be used to unschedule the task or
            retrieve the results of the callback after the task has
            been executed.
            
        """
        if args is None:
            args = ()

        if kwargs is None:
            kwargs = {}

        task = ScheduledTask(callback, args, kwargs)

        heap = self.__heap
        with self.__heap_lock:
            needs_start = len(heap) == 0
            item = (priority, self.__counter.next(), task)
            heappush(heap, item)

        if needs_start:
            self.call_on_main(self.__next_task)
        
        return task

    def has_pending_tasks(self):
        """ Returns True if the scheduler has pending tasks, False 
        otherwise.

        """
        with self.__heap_lock:
            has_pending = len(self.__heap) > 0
        return has_pending

    #--------------------------------------------------------------------------
    # Abstract API 
    #--------------------------------------------------------------------------
    @abstractmethod
    def initialize(self, *args, **kwargs):
        """ An abstract method which must be implemented by a toolkit
        specific subclass.

        The toolkit implementation of this method should initialize
        an necessary application object and the set the _initialized
        flag to True. It should *not* start the event loop. If the
        application object is already created, it should be a no-op.

        The args and kwargs should be passed, as necessary, to the
        application object constructor.

        """
        raise NotImplementedError
    
    @abstractmethod
    def start_event_loop(self):
        """ An abstract method which must be implemented by a toolkit
        specific subclass.

        The toolkit implementation of this method should start the 
        underlying event loop, or do nothing if it is already started.
        It should raise a RuntimeError if the application object is
        not yet created.

        """
        raise NotImplementedError
    
    @abstractmethod
    def event_loop_running(self):
        """ An abstract method which must be implemented by a toolkit
        specific subclass.

        The toolkit implementation of this method should return True
        if the main event loop is running, False otherwise.

        """
        raise NotImplementedError

    @abstractmethod
    def app_object(self):
        """ An abstract method which must be implemented by a toolkit
        specific subclass.

        The toolkit implementation of this method should return the
        underlying application object, or None if one does not exist.

        """
        raise NotImplementedError

    @abstractmethod    
    def is_main_thread(self):
        """ An abstract method which must be implemented by a toolkit
        specific subclass.

        The toolkit implementation of this method should return True
        if the method was called from the main gui event thread, or
        False otherwise.

        """
        raise NotImplementedError

    @abstractmethod
    def call_on_main(self, callback, *args, **kwargs):
        """ An abstract method which must be implemented by a toolkit
        specific subclass. 

        The toolkit implementation of this method should invoke the 
        given callable in the main gui event thread at some point 
        in the future.

        Parameters
        ----------
        callback : callable
            The callable object to execute at some point in the future.

        *args
            Any positional arguments to pass to the callback.

        **kwargs
            Any keyword arguments to pass to the callback.

        """
        raise NotImplementedError

    @abstractmethod
    def timer(self, ms, callback, *args, **kwargs):
        """ An abstract method which must be implemented by a toolkit
        specific subclass. 

        The toolkit implementation of this method should invoke the 
        given callable in the main gui event thread at some point 
        in the future.

        Parameters
        ----------
        callback : callable
            The callable object to execute at some point in the future.

        *args
            Any positional arguments to pass to the callback.

        **kwargs
            Any keyword arguments to pass to the callback.

        """
        raise NotImplementedError
    
    @abstractmethod
    def process_events(self):
        """ An abstract method which must be implemented by a toolkit
        specific subclass.

        The toolkit implementation of this method should process all
        pending events in its event queue.

        """
        raise NotImplementedError




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance

from .toggle_control import ToggleControl, AbstractTkToggleControl

from ..core.trait_types import CoercingInstance
from ..layout.geometry import Size
from ..noncomponents.abstract_icon import AbstractTkIcon


class AbstractTkToggleButton(AbstractTkToggleControl):
    """ The abstract toolkit ToggleButton interface.

    """
    @abstractmethod
    def shell_icon_changed(self, icon):
        """ The change handler for the 'icon' attribute on the shell
        component.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_icon_size_changed(self, icon_size):
        """ The change handler for the 'icon_size' attribute on the 
        shell component.

        """
        raise NotImplementedError


class ToggleButton(ToggleControl):
    """ A toggle button widget.

    A toggle button stays down when pressed and returns to the up 
    position when pressed again. It encapsulates the functionality of
    a CheckBox in the form of a PushButton.

    Use a toggle button when it's necessary to toggle a boolean value
    independent of any other widgets in the group. For allowing the 
    toggling of only one value in a group of values, use a group of 
    radio buttons.

    See Also
    --------
    ToggleControl

    """
    #: The an icon to display in the button.
    icon = Instance(AbstractTkIcon)

    #: The size of the icon.
    icon_size = CoercingInstance(Size)

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkToggleButton)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import ABCMeta, abstractmethod, abstractproperty

from traits.api import Property, Instance

from ..core.base_component import BaseComponent


class AbstractTkBaseWidgetComponent(object):
    """ The abstract toolkit BaseWidgetComponent interface.

    A toolkit component is responsible for handling changes on a shell 
    Component and proxying those changes to and from its internal guit
    toolkit widget.

    """
    __metaclass__ = ABCMeta
    
    @abstractproperty
    def toolkit_widget(self):
        """ An abstract property that should return the gui toolkit 
        widget being managed by the object.

        """
        raise NotImplementedError

    @abstractproperty
    def shell_obj(self):
        """ An abstract property that should *get* and *set* a reference
        to the shell object (an instance of Component). 

        It is suggested that the implementation maintain only a weakref
        to the shell object in order to avoid reference cycles.

        """
        raise NotImplementedError

    @abstractmethod
    def create(self, parent):
        """ Create the underlying implementation object. 

        This method is called after the reference to the shell object
        has been set and is called in depth-first order. This means
        that by the time this method is called, the logical parent
        of this instance has already been created.

        Parameters
        ----------
        parent : toolkit widget or None
            The toolkit widget that will be the parent for the internal
            widget.

        """
        raise NotImplementedError
    
    @abstractmethod
    def initialize(self):
        """ Initialize the implementation object.

        This method is called after 'create' in depth-first order. This
        means that all other implementations in the tree will have been
        created so that intialization can depend on the existence of 
        other implementation objects.

        Rather than raise the more typical NotImplementedError, this
        abstractmethod is no-op so that implementors with complex
        multiple inheritence heierarhies can use super() all the 
        way to the top.

        """
        pass

    @abstractmethod
    def bind(self):
        """ Called after 'initialize' in order to bind event handlers.

        At the time this method is called, the entire tree of ui
        objects will have been initialized. The intention of this 
        method is delay the binding of event handlers until after
        everything has been intialized in order to mitigate extraneous
        event firing.

        Rather than raise the more typical NotImplementedError, this
        abstractmethod is no-op so that implementors with complex
        multiple inheritence heierarhies can use super() all the 
        way to the top.

        """
        pass

    @abstractmethod
    def destroy(self):
        """ Called when the underlying widget should be destroyed.

        This method is called before the child shell object is removed
        from its parent. This enables a toolkit backend to ensure that
        the underlying toolkit widget objects are properly removed 
        from the widget tree before the abstract wrapper is discarded.
                
        """
        raise NotImplementedError


#------------------------------------------------------------------------------
# BaseWidgetComponent
#------------------------------------------------------------------------------
class BaseWidgetComponent(BaseComponent):
    """ A BaseComponent subclass which adds support for a gui toolkit 
    specific backend object. This class represents the most basic 
    component in Enaml that drives a gui toolkit object. The gui toolkit 
    object at this level is not necessarily a widget which can paint on
    the screen.

    """
    #: A read-only property that returns the toolkit specific widget
    #: being managed by the abstract widget.
    toolkit_widget = Property

    #: The toolkit specific object that implements the behavior of
    #: this component and manages the gui toolkit object. Subclasses
    #: should redefine this trait to specify the specialized type of
    #: abstract_obj that is accepted.
    abstract_obj = Instance(AbstractTkBaseWidgetComponent)

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    def _get_toolkit_widget(self):
        """ The property getter for the 'toolkit_widget' attribute.

        """
        return self.abstract_obj.toolkit_widget
    
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def _setup_create_widgets(self, parent):
        """ A reimplemented parent class setup method which creates the
        underlying toolkit widgets.

        """
        self.abstract_obj.create(parent)
        self_widget = self.toolkit_widget
        super(BaseWidgetComponent, self)._setup_create_widgets(self_widget)

    def _setup_init_widgets(self):
        """ A reimplemented parent class setup method which initializes
        the underyling toolkit widgets.

        """
        self.abstract_obj.initialize()
        super(BaseWidgetComponent, self)._setup_init_widgets()
    
    def _setup_bind_widgets(self):
        """ A reimplemented parent class setup method which binds the
        event handlers for the internal toolkit widgets.

        """
        self.abstract_obj.bind()
        super(BaseWidgetComponent, self)._setup_bind_widgets()
    
    def _setup_listeners(self):
        """ A reimplemented parent class setup method which sets up
        the trait listeners for the abstract object.

        """
        self.add_trait_listener(self.abstract_obj, 'shell')
        super(BaseWidgetComponent, self)._setup_listeners()

    #--------------------------------------------------------------------------
    # Teardown Methods
    #--------------------------------------------------------------------------
    def destroy(self):
        """ Overridden parent class destruction method method that adds 
        additional logic to destroy the underlying toolkit widget. 

        The destruction happens in dual pass top-down, then bottom-up.
        On the top-down pass, the traits listeners for the abstract obj
        are unhooked so that no events get fired during destruction. On
        the bottom up pass, the abstract obj is destroyed and its ref
        set to None.

        """
        # Remove the abstract object as a trait listener so that it
        # does not try to update after destroying its internal widget.
        self.remove_trait_listener(self.abstract_obj, 'shell')

        # Traverse down the tree and have the children destroy themselves.
        super(BaseWidgetComponent, self).destroy()
        
        # On pass back up the tree, destroy the abstract_obj.
        self.abstract_obj.destroy()
        self.abstract_obj = None




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
""" Subclasses of standard QWidgets that add a `resized` signal that will 
pass on notifications of resize events to Enaml.

"""
from abc import ABCMeta
import os

from .qt import QtCore, QtGui


#: A private flag which is used at startup time to enable debug painting
#: for the Qt resizable widgets.
_debug = os.environ.get('ENAML_DEBUG_PAINT', False)


class QResizingWidget(object):
    """ An abstract base class for testing if a QWidget subclass exposes
    a `resized` signal. Virtual subclasses must explicitly register 
    themselves with the .register() method.

    """
    __metaclass__ = ABCMeta


def _resizable(cls):
    """ A private class decorator/factory which takes a QWidget subclass 
    and adds code which converts its resize events into a 'resized' 
    signal. If the debug paint environment flag is set, it adds the 
    custom painting code as well.

    Paramters
    ---------
    cls : QWidget subclass
        The QWidget subclass to which resizing functionality is added.
    
    Returns
    -------
    result : QWidget subclass
        A new subclass with the resizing functionality implemented.
    
    """
    if _debug:

        class QResizable(cls):
            """ A QWidget subclass that passes its resize events back 
            to Enaml through a Qt signal.

            """
            #: A signal which is emitted on a resize event.
            resized = QtCore.Signal()

            def resizeEvent(self, event):
                """ Converts a resize event in into a signal.

                """
                super(QResizable, self).resizeEvent(event)
                self.resized.emit()
            
            def paintEvent(self, event):
                """ Paints debug layout rects of all the children during
                a paint event.

                """
                super(QResizable, self).paintEvent(event)
                qp = QtGui.QPainter()
                qp.begin(self)
                try:
                    qp.setPen(QtGui.QColor(255, 0, 0))
                    for child in self.children():
                        if isinstance(child, QtGui.QWidget):
                            layout_item = QtGui.QWidgetItem(child)
                            geom = layout_item.geometry()
                            qp.drawRect(geom)
                finally:
                    qp.end()
            
        QResizable.__name__ = cls.__name__ + '_debug_resizable'
                
    else:
        
        class QResizable(cls):
            """ A QWidget subclass that passes its resize events back 
            to Enaml through a Qt signal.

            """
            #: A signal which is emitted on a resize event.
            resized = QtCore.Signal()

            def resizeEvent(self, event):
                """ Converts a resize event in into a signal.

                """
                super(QResizable, self).resizeEvent(event)
                self.resized.emit()
    
        QResizable.__name__ = cls.__name__ + '_resizable' 
          
    QResizingWidget.register(QResizable)

    return QResizable


QResizingFrame = _resizable(QtGui.QFrame)


QResizingDialog = _resizable(QtGui.QDialog)


QResizingMainWindow = _resizable(QtGui.QMainWindow)


QResizingGroupBox = _resizable(QtGui.QGroupBox)


QResizingScrollArea = _resizable(QtGui.QScrollArea)


QResizingTabWidget = _resizable(QtGui.QTabWidget)


QResizingSplitter = _resizable(QtGui.QSplitter)





#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_test_assistant import WXTestAssistant, skip_nonwindows
from .. import check_box

@skip_nonwindows
class TestWXCheckBox(WXTestAssistant, check_box.TestCheckBox):
    """ WXCheckbox tests. """

    def get_text(self, widget):
        """ Returns the text from the tookit widget.

        """
        return widget.GetLabel()

    def get_checked(self, widget):
        """ Returns the checked status from the tookit widget.

        """
        return widget.GetValue()

    def checkbox_pressed(self, widget):
        """ Press the checkbox programmatically.

        """
        self.send_wx_event(widget, wx.EVT_LEFT_DOWN)

    def checkbox_released(self, widget):
        """ Release the button programmatically.

        """
        self.send_wx_event(widget, wx.EVT_LEFT_UP)
        self.send_wx_event(widget, wx.EVT_LEAVE_WINDOW)

    def checkbox_toggle(self, widget):
        """ Toggle the button programmatically.

        """
        self.send_wx_event(widget, wx.EVT_CHECKBOX)



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_toggle_control import WXToggleControl

from ...components.toggle_button import AbstractTkToggleButton


class WXToggleButton(WXToggleControl, AbstractTkToggleButton):
    """ A wxPython implementation of ToggleButton.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.ToggleButton.

        """
        self.widget = wx.ToggleButton(parent)

    def bind(self):
        """ Binds the event handlers for the check box.

        """
        super(WXToggleButton, self).bind()
        widget = self.widget
        widget.Bind(wx.EVT_TOGGLEBUTTON, self.on_toggled)
        widget.Bind(wx.EVT_LEFT_DOWN, self.on_pressed)
        widget.Bind(wx.EVT_LEAVE_WINDOW, self.on_released)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_icon_changed(self, icon):
        """ The change handler for the 'icon' attribute.

        This is currently not implemented because the standard 
        wx.ToggleButton does not support images.

        """
        pass

    def shell_icon_size_changed(self, icon_size):
        """ The change handler for the 'icon_size' attribute.

        This is currently not implemented because the standard 
        wx.ToggleButton does not support images.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_test_assistant import WXTestAssistant, skip_nonwindows

from .. import spin_box


@skip_nonwindows
class TestWxSpinBox(WXTestAssistant, spin_box.TestSpinBox):
    """ WXSpinBox tests. """

    def get_value(self, widget):
        """ Get a spin box's value.

        """
        return widget.GetValue()

    def get_low(self, widget):
        """ Get a spin box's minimum value.

        """
        return widget.GetLow()

    def get_high(self, widget):
        """ Get a spin box's maximum value.

        """
        return widget.GetHigh()

    def get_step(self, widget):
        """ Get a spin box's step size.

        """
        return widget.GetStep()

    def get_wrap(self, widget):
        """ Check if a spin box wraps around at the edge values.

        """
        return widget.GetWrap()

    def get_text(self, widget):
        """ Get the text displayed in a spin box.

        """
        return widget._value_string

    def spin_up_event(self, widget):
        """ Simulate a click on the 'up' spin button.

        """
        event = wx.SpinEvent(wx.EVT_SPIN_UP.typeId)
        widget.OnSpinUp(event)
        self.process_wx_events(wx.GetApp())

    def spin_down_event(self, widget):
        """ Simulate a click on the 'down' spin button.

        """
        event = wx.SpinEvent(wx.EVT_SPIN_DOWN.typeId)
        widget.OnSpinDown(event)
        self.process_wx_events(wx.GetApp())




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import weakref

import wx

from .wx_action import wxEnamlAction
from .wx_widget_component import WXWidgetComponent

from ...components.menu import AbstractTkMenu


# A note to future developers who consider trying to code the event 
# handling logic to implement the about_to_show/hide functionality:
# Don't. I wasted 8 hours of my life trying to get it to work, only
# to have wx get in my way at every step. For example, in order to 
# actually receive EVT_MENU_OPEN/CLOSE events on Windows, one has to 
# bind to the events on the parent wxFrame rather than the menus 
# themselves. Ok fine. You can do that, only to have the wxMenuEvent 
# return None from GetMenu() at various times. For example, it will 
# always give None from an EVT_MENU_CLOSE, and it will give None from 
# an EVT_MENU_OPEN on a submenu. This makes it utterly pointless to 
# try to do any handling since one has no idea which menu actually 
# triggered the event. So until wx actually fixes the menu events 
# (don't hold your breath, the issue has been known since 2006), any
# effort to get it to work won't be worth it. Just use Qt.
class wxEnamlMenu(wx.Menu):
    """ A wx.Menu subclass which adapts the wx menu api to work with the
    menu semantics of Enaml.

    """
    def __init__(self, parent, *args, **kwargs):
        """ Initialize a wxEnamlMenu.

        Parameters
        ----------
        parent : wxEnamlMenuBar or wxEnamlMenu
            The logic parent of this menu.

        *args
            Additional positional arguments to pass to the wx.MenuBar
            constructor.
        
        **kwargs
            Additional keyword arguments to pass to the wx.MenuBar
            constructor.
        
        """
        super(wxEnamlMenu, self).__init__(*args, **kwargs)
        self._parent = weakref.ref(parent)
        self._enabled = True

    def GetParent(self):
        """ Returns the parent of this menu.

        """
        return self._parent()

    def RemoveAll(self):
        """ Removes all of the menu items from the menu. It does not 
        destroy any of the menu items.

        """
        items = list(self.GetMenuItems())
        for item in items:
            self.RemoveItem(item)

    def IsEnabled(self):
        """ Returns whether or not the menu is enabled. Note that this
        overrides a parent class method of the same name but with a 
        different call signature and different semantics.

        """
        return self._enabled

    def Enable(self, enable):
        """ Enables or disables the menu. Note that this overrides a 
        parent class method of the same name but with a different call 
        signature and different semantics.

        """
        parent = self._parent()
        if parent is not None and parent.IsEnabled():
            if isinstance(parent, wx.MenuBar):
                for idx, (menu, _) in enumerate(parent.GetMenus()):
                    if menu == self:
                        parent.EnableTop(idx, enable)
                        break
            elif isinstance(parent, wx.Menu):
                for item in parent.GetMenuItems():
                    if item.GetSubMenu() == self:
                        item.Enable(enable)
                        break
            else:
                msg = 'Invalid parent for wxEnamlMenu %s'
                raise TypeError(msg % parent)
        self._enabled = enable

    def SetTitle(self, title):
        """ Sets the title for the menu. Note that this overrides a 
        parent class method of the same name but with a different call 
        signature and different semantics.

        """
        # We do *not* want to use the default SetTitle implementation 
        # here since it doesn't change the title of the entry in the 
        # menubar. Instead, it adds a non-functional bold-faced entry 
        # into the menu itself, followed by a separator and the rest 
        # of the menu items. The docs say its meant for popup menus,
        # but I find it just confuses the api and serves no purpose.
        parent = self._parent()
        if parent is not None:
            if isinstance(parent, wx.MenuBar):
                for idx, (menu, _) in enumerate(parent.GetMenus()):
                    if menu == self:
                        parent.SetMenuLabel(idx, title)
                        break
            elif isinstance(parent, wx.Menu):
                for item in parent.GetMenuItems():
                    if item.GetSubMenu() == self:
                        item.SetItemLabel(title)
                        break
            else:
                msg = 'Invalid parent for wxEnamlMenu %s'
                raise TypeError(msg % parent)

    def Destroy(self):
        """ An overridden parent class method which removes this menu
        from its parent before invoking the actual destructor.

        """
        if self:
            parent = self._parent()
            if parent is not None:
                if isinstance(parent, wx.MenuBar):
                    for idx, (menu, _) in enumerate(parent.GetMenus()):
                        if menu == self:
                            parent.Remove(idx)
                            break
                elif isinstance(parent, wx.Menu):
                    for item in parent.GetMenuItems():
                        if item.GetSubMenu() == self:
                            parent.RemoveItem(item)
                            break
                else:
                    msg = 'Invalid parent for wxEnamlMenu %s'
                    raise TypeError(msg % parent)
            super(wxEnamlMenu, self).Destroy()


class WXMenu(WXWidgetComponent, AbstractTkMenu):
    """ A Wx implementation of Menu.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Create the underlying QMenu object.

        """
        self.widget = wxEnamlMenu(parent)

    def initialize(self):
        """ Initialize the underlying QMenu object.

        """
        super(WXMenu, self).initialize()
        # We don't need to initialize the title for a wx Menu since 
        # that is handled by its parent when its added to the menubar
        # or menu. However, if the title changes, it is updated by 
        # the change handler defined below.
        self.set_title(self.shell_obj.title)
        self.update_contents()
    
    # There is nothing to bind on a Menu since the wx Menu doesn't
    # support proper show/hide notification. See the block comment
    # above the definition of wxEnamlMenu for more explanation.

    #--------------------------------------------------------------------------
    # Change Handlers 
    #--------------------------------------------------------------------------
    def shell_title_changed(self, text):
        """ The change handler for the 'title' attribute on the shell
        object.

        """
        self.set_title(text)
        
    def shell_contents_changed(self, contents):
        """ The change handler for the 'contents' attribute on the shell
        object. 

        """
        self.update_contents()

    #--------------------------------------------------------------------------
    # Signal Handlers
    #--------------------------------------------------------------------------
    def on_about_to_show(self, event):
        """ A signal handler for the 'aboutToShow' signal of the QMenu.

        """
        self.shell_obj.about_to_show()

    def on_about_to_hide(self, event):
        """ A signal handler for the 'aboutToHide' signal of the QMenu.

        """
        self.shell_obj.about_to_hide()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def update_contents(self):
        """ Updates the contents of the Menu.

        """
        # We can't reason reliably on the proper ordering of menus
        # in the menu bar because of the variety of possible uses
        # of the Include component. Instead, we just remove the 
        # existing menu items, and then add the new ones.
        widget = self.widget
        widget.RemoveAll()
        for item in self.shell_obj.contents:
            child_widget = item.toolkit_widget
            if isinstance(child_widget, wxEnamlMenu):
                widget.AppendSubMenu(child_widget, item.title)
            elif isinstance(child_widget, wxEnamlAction):
                child_widget.Install()
            else:
                msg = 'Invalid child for wxEnamlMenu: %s'
                raise TypeError(msg % child_widget)

    def set_title(self, title):
        """ Sets the title of the wxMenu object.

        """
        self.widget.SetTitle(title)
    
    def set_visible(self, visible):
        """ An overridden parent class method. A wxMenu cannot change 
        its visibility.

        """
        pass
    
    def set_bgcolor(self, bgcolor):
        """ An overridden parent class method. A wxMenu cannot change 
        its bgcolor.

        """
        pass
    
    def set_fgcolor(self, fgcolor):
        """ An overridden parent class method. A wxMenu cannot change 
        its fgcolor.

        """
        pass
    
    def set_font(self, font):
        """ An overridden parent class method. A wxMenu cannot change 
        its font.

        """
        pass

    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def popup(self, blocking=True):
        """ Pops up the menu on the toplevel widget using the current 
        position of the cursor. The blocking flag is ignored since it
        is not supported by wx. All popup menus block on wx.

        """
        widget = self.widget
        frame = widget
        while True:
            if frame is None or isinstance(frame, wx.Frame):
                break
            frame = frame.GetParent()
        if frame is None:
            windows = wx.GetTopLevelWindows()
            if len(windows) == 0:
                msg = 'Cannot find a suitable toplevel window for popup menu'
                raise ValueError(msg)
            frame = windows[0]
        frame.PopupMenu(self.widget)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_toggle_control import WXToggleControl

from ...components.check_box import AbstractTkCheckBox


class WXCheckBox(WXToggleControl, AbstractTkCheckBox):
    """ A wxPython implementation of CheckBox.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.CheckBox.

        """
        self.widget = wx.CheckBox(parent)

    def bind(self):
        """ Binds the event handlers for the check box.

        """
        super(WXCheckBox, self).bind()
        widget = self.widget
        widget.Bind(wx.EVT_CHECKBOX, self.on_toggled)
        widget.Bind(wx.EVT_LEFT_DOWN, self.on_pressed)
        widget.Bind(wx.EVT_LEAVE_WINDOW, self.on_released)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import Instance, Property, Tuple, Enum, on_trait_change

from .widget_component import WidgetComponent, AbstractTkWidgetComponent

from ..layout.constrainable import Constrainable
from ..layout.geometry import Rect


PolicyEnum = Enum('ignore', 'weak', 'medium', 'strong', 'required')


#------------------------------------------------------------------------------
# Abstract Toolkit Constraints Widget Interface
#------------------------------------------------------------------------------
class AbstractTkConstraintsWidget(AbstractTkWidgetComponent):
    """ The abstract toolkit ConstraintsWidget interface.

    """
    pass


#------------------------------------------------------------------------------
# ConstraintsWidget
#------------------------------------------------------------------------------
class ConstraintsWidget(WidgetComponent, Constrainable):
    """ A WidgetComponent subclass which mixes in the Constrainable
    interface and adds some additional hints for being managed by 
    a Container.

    """
    #: How strongly a component hugs it's contents' width. Valid strengths
    #: are 'weak', 'medium', 'strong', 'required' and 'ignore'. Default is 
    #: 'strong'. This trait should be overridden on a per-control basis to
    #: specify a logical default for the given control.
    hug_width = PolicyEnum('strong')

    #: How strongly a component hugs it's contents' height. Valid strengths
    #: are 'weak', 'medium', 'strong', 'required' and 'ignore'. Default is
    #: 'strong'. This trait should be overridden on a per-control basis to 
    #: specify a logical default for the given control.
    hug_height = PolicyEnum('strong')

    #: The combination of (hug_width, hug_height).
    hug = Property(
        Tuple(PolicyEnum, PolicyEnum), depends_on='hug_width, hug_height',
    )

    #: How strongly a component resists clipping its contents. Valid 
    #: strengths are 'weak', 'medium', 'strong', 'required' and 'ignore'. 
    #: The default is 'strong' for width.
    resist_clip_width = PolicyEnum('strong')

    #: How strongly a component resists clipping its contents. Valid 
    #: strengths are 'weak', 'medium', 'strong', 'required' and 'ignore'. 
    #: The default is 'strong' for height.
    resist_clip_height = PolicyEnum('strong')

    #: The combination of (resist_clip_width, resist_clip_height).
    resist_clip = Property(
        Tuple(PolicyEnum, PolicyEnum),
        depends_on='resist_clip_width, resist_clip_height',
    )

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkConstraintsWidget)

    #--------------------------------------------------------------------------
    # Property Getters and Setters
    #--------------------------------------------------------------------------
    def _get_hug(self):
        """ Property getter for the 'hug' property.

        """
        return (self.hug_width, self.hug_height)

    def _set_hug(self, value):
        """ Property setter for the 'hug' property.

        """
        width, height = value
        self.trait_set(hug_width=width, hug_height=height)

    def _get_resist_clip(self):
        """ Property getter for the 'resist_clip' property.

        """
        return (self.resist_clip_width, self.resist_clip_height)

    def _set_resist_clip(self, value):
        """ Property setter for the 'resist_clip' property.

        """
        width, height = value
        self.trait_set(resist_clip_width=width, resist_clip_height=height)

    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    @on_trait_change('hug, resist_clip')
    def _on_constraints_widget_deps_changed(self):
        """ A handler which requests a relayout from the *parent* when
        the 'hug' or 'resist_clip' values change, provided that the 
        component is initialized. 

        Note that it is critical that the request is made to the parent
        since the component itself has not use for its size hint in 
        its own internal layout calculations.

        """
        if self.initialized:
            parent = self.parent
            if parent is not None:
                parent.request_relayout()
    
    #--------------------------------------------------------------------------
    # Constraint Handlers
    #--------------------------------------------------------------------------
    def size_hint_constraints(self):
        """ Returns the list of constraints relating to the size hint 
        of this layout component. If the size hint in a given dimension
        is negative, then no constraints will be generated for that
        dimension.

        """
        cns = []
        push = cns.append

        width_hint, height_hint = self.size_hint()
        width = self.width
        height = self.height
        hug_width = self.hug_width
        hug_height = self.hug_height
        resist_clip_width = self.resist_clip_width
        resist_clip_height = self.resist_clip_height

        if width_hint >= 0:
            if hug_width != 'ignore':
                cn = (width == width_hint) | hug_width
                push(cn)
            if resist_clip_width != 'ignore':
                cn = (width >= width_hint) | resist_clip_width
                push(cn)
        
        if height_hint >= 0:
            if hug_height != 'ignore':
                cn = (height == height_hint) | hug_height
                push(cn)
            if resist_clip_height != 'ignore':
                cn = (height >= height_hint) | resist_clip_height
                push(cn)

        return cns
    
    #--------------------------------------------------------------------------
    # Constrainable Interface
    #--------------------------------------------------------------------------
    def update_layout_geometry(self, dx, dy):
        """ An implementation of the required abstract method of the 
        Constrainable mixin. This computes the new layout geometry
        rect and updates the underlying widget.

        """
        x = int(round(self.left.value))
        y = int(round(self.top.value))
        width = int(round(self.width.value))
        height = int(round(self.height.value))
        rect = Rect(x - dx, y - dy, width, height)
        self.set_layout_geometry(rect)
        return (x, y)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .abstract_validator import AbstractValidator


class CoercingValidator(AbstractValidator):
    """ A concrete implementation of AbstractValidator which uses various
    callables to convert, coerce, and format between values and text. 

    This class serves as a base class for several other validators, such
    as the number and date/time validators. It does not on its own use 
    the provided locale object for conversions.

    A CoercingValidator instance will, by default, consider all inputs
    as valid and format all values according to their default unicode
    representation.

    """
    def __init__(self, converter=None, coercer=None, formatter=None, **kwargs):
        """ Initialize a CoercingValidator.

        Parameters
        ----------
        converter : callable or None, optional
            If provided, a callable which accepts the string value
            and returns a value to pass the coercing function. If not
            provided, a suitable default is used. Defaults to None.
        
        coercer : callable or None, optional
            If provided, a callable which accepts the evaluated value
            and returns a new coerced value to use for verifying 
            validity. If not provided, a suitable default is used. 
            Defaults to None.
        
        formatter : callable or None, optional
            If provided, a callable which accepts the context-dependent
            model value and returns a unicode string for display. If 
            not provided, a suitable default is used. Defaults to None.
        
        **kwargs
            The keyword arguments necessary to initialize an instance
            of AbstractValidator.

        """
        super(CoercingValidator, self).__init__(**kwargs)
        if converter is not None:
            self._convert = converter
        if coercer is not None:
            self._coerce = coercer
        if formatter is not None:
            self._format = formatter
    
    def _convert(self, text):
        """ The default method to convert the string to a number. This
        may be overridden by subclasses or through instance-specific
        attributes.
        
        """
        return text
         
    def _coerce(self, value):
        """ The default method to coerce the given value into a value 
        to test for validity. This may be overridden by subclasses or 
        through instance-specific attributes.

        """
        return value

    def _format(self, value):
        """ The default method to format the given value to a unicode
        string for display. This may be overridden by subclasses or 
        through instance-specific attributes.

        """
        return unicode(value)

    def _validate(self, text):
        """ An internal validation method which returns a tuple of
        information related to the validation.

        This method is useful for subclasses to eliminate redundant 
        coercion code in the validate method.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.
        
        Returns
        -------
        result : 2-tuple
            A 2-tuple of (<validation_result>, <value|None>). If the
            conversion and coercion are successful, then the first item 
            will be the ACCEPTABLE constant and the second item will be 
            the converted and coerced value. Otherwise, the first item 
            will be the INVALID constant and the second item will be None.
        
        """
        try:
            value = self._convert(text)
        except Exception:
            return (self.INVALID, None)
        
        try:
            value = self._coerce(value)
        except Exception:
            return (self.INVALID, None)
        
        return (self.ACCEPTABLE, value)
    
    def validate(self, text):
        """ Validates the input text against the rules of the validator.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.
        
        Returns
        -------
        result : int
            One of the class attribute constants INVALID or ACCEPTABLE.
        
        Notes
        -----
        The string is INVALID under the following conditions:
            - The string fails to convert.
            - The converted value fails to coerce.
            - The coerced value is not the proper type.

        The string is ACCEPTABLE for all other conditions.

        """
        return self._validate(text)[0]

    def convert(self, text):
        """ Converts the user input text into an appropriate value or
        raises a ValueError if the conversion fails.

        This method is called only if the 'validate' method returns the
        ACCEPTABLE class attribute constant.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.

        Returns
        -------
        result : object
            An appropriately converted object.
        
        """
        try:
            return self._coerce(self._convert(text))
        except Exception as e:
            msg = "Failed to convert %r. Original exception was %s."
            raise ValueError(msg % (text, e))

    def format(self, value):
        """ Convert the input value into a string which is appropriate
        for display.

        Parameters
        ----------
        value : object
            The context-dependent input value

        Returns
        -------
        result : unicode string
            A formatted unicode string for display.

        """
        return unicode(self._format(value))

    def normalize(self, text):
        """ Attempts to normalize the given text by converting it to the 
        numeric form, then back to unicode using the formatter. Returns 
        the original text if the conversion fails.

        """
        try:
            return self.format(self._coerce(self._convert(text)))
        except Exception:
            return text




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_toggle_control import QtToggleControl

from ...components.radio_button import AbstractTkRadioButton


class QtRadioButton(QtToggleControl, AbstractTkRadioButton):
    """ A Qt implementation of RadioButton.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying Qt widget.

        """
        self.widget = QtGui.QRadioButton(parent)

    def bind(self):
        """ Binds the event handlers for the radio button.

        """
        super(QtRadioButton, self).bind()
        widget = self.widget
        widget.toggled.connect(self.on_toggled)
        widget.pressed.connect(self.on_pressed)
        widget.released.connect(self.on_released)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from ...core.constructor import Constructor


def importer(module_path, name):
    def _importer():
        mod = __import__(module_path, fromlist=[name])
        try:
            res = getattr(mod, name)
        except AttributeError:
            raise ImportError('Cannot import name %s' % name)
        return res
    return _importer


def shell_loader(base_path):
    """ Get the shell class loader for the given name.

    Parameters
    ----------
    base_path : str
        The lowercase-with-underscores name of the module containing the 
        shell class.

    Returns
    -------
    c_name : str
        The name of the shell class.

    shell_loader : callable
        The loader function for the shell class.

    """
    c_module_path = 'enaml.components.' + base_path
    c_name = ''.join(part.capitalize() for part in base_path.split('_'))
    shell_loader = importer(c_module_path, c_name)
    return c_name, shell_loader


def abstract_loader(c_name, base_path):
    """ Get the abstract class loader for the given name.

    Parameters
    ----------
    c_name : str
        The TitleCase class name of the shell class.

    base_path : str
        The lowercase-with-underscores name of the module containing the 
        shell class.

    Returns
    -------
    abstract_loader : callable
        The loader function for the abstract class.

    """
    t_module_path = 'enaml.backends.qt.' + 'qt_' + base_path
    t_name = 'Qt' + c_name
    abstract_loader = importer(t_module_path, t_name)
    return abstract_loader


def constructor(base_path):
    """ A factory function which understands our name mangling and will
    create a constructor instance. Returns tuple of (name, ctor) where
    name is a string that can be used by toolkit to refer to the ctor
    in the enaml source code.

    """
    c_name, shell_ldr = shell_loader(base_path)
    abstract_ldr = abstract_loader(c_name, base_path)
    ctor = Constructor(shell_ldr, abstract_ldr)
    return c_name, ctor


base_sel_model = constructor('base_selection_model')[1]
row_sel_model = base_sel_model.clone(shell_loader('row_selection_model')[1])


def tui_imp():
    from ...components.traitsui_item import TraitsUIItem
    return TraitsUIItem


def qt_tui_imp():
    from .qt_traitsui_item import QtTraitsUIItem
    return QtTraitsUIItem


tui_item = Constructor(tui_imp, qt_tui_imp)


def canvas_imp():
    from ...components.canvas import Canvas
    return Canvas


def qt_canvas_imp():
    from .qt_canvas import QtCanvas
    return QtCanvas


canvas_ctor = Constructor(canvas_imp, qt_canvas_imp)


QT_CONSTRUCTORS = dict((
    constructor('window'),
    constructor('main_window'),
    constructor('container'),
    constructor('dialog'),
    constructor('directory_dialog'),
    constructor('calendar'),
    constructor('check_box'),
    constructor('combo_box'),
    constructor('field'),
    constructor('file_dialog'),
    constructor('html'),
    constructor('image_view'),
    constructor('label'),
    constructor('push_button'),
    constructor('radio_button'),
    constructor('toggle_button'),
    constructor('slider'),
    constructor('spin_box'),
    ('TraitsUIItem', tui_item),
    constructor('enable_canvas'),
    constructor('text_editor'),
    constructor('list_view'),
    constructor('table_view'),
    constructor('tree_view'),
    constructor('date_edit'),
    constructor('datetime_edit'),
    constructor('form'),
    constructor('group_box'),
    constructor('scroll_area'),
    constructor('progress_bar'),
    constructor('tab_group'),
    constructor('tab'),
    constructor('splitter'),
    constructor('float_slider'),
    ('BaseSelectionModel', base_sel_model),
    ('RowSelectionModel', row_sel_model),
    constructor('menu_bar'),
    constructor('menu'),
    constructor('action'),
    ('Canvas', canvas_ctor),
    constructor('dock_pane'),
))




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_constraints_widget import WXConstraintsWidget

from ...components.scroll_area import AbstractTkScrollArea


# As mentioned in the notes of the docstring for WXScrollArea, we
# don't support the 'always_on' policy of the scrollbars. Since
# we only have to support 'off' or 'auto' it's easiest to use this
# mapping to convert straight from policy values into a respective
# scroll rate. A rate of Zero causes wx not to show the scroll bar.
# A positive rate indicates to scroll that many pixels per event.
# We set the rate to 1 to have smooth scrolling. Wx doesn't make a
# distinction between scroll events caused by the mouse or scrollbar
# and those caused by clicking the scroll buttons (ala qt), and thus
# this rate applies the same to all of those events. Since we expect
# that clicking on a scroll button happens much more infrequently 
# than scrolling by dragging the scroll bar, we opt for a lower rate
# in order to get smooth drag scrolling and sacrifice some usability
# on the scroll buttons.
SCROLLBAR_POLICY_MAP = {
    'as_needed': 1,
    'always_off': 0,
    'always_on': 1,
}


class WXScrollAreaSizer(wx.PySizer):
    """ A custom wx Sizer for use in the WXScrollArea. This sizer expands
    its child to fit the allowable space, regardless of the settings on
    the child settings.

    There can only be one component in this sizer at a time and it 
    should be added via the .Add(...) method. Old items will be removed 
    automatically (but not destroyed).

    """
    #: The child component which is manipulated by the sizer.
    _child = None

    def Add(self, component):
        """ Adds the given child component to the sizer, removing the 
        old widget if present. The old widget is not destroyed.

        """
        self.Clear(deleteWindows=False)
        self._child = component
        if component is not None:
            child_widget = component.toolkit_widget
            res = super(WXScrollAreaSizer, self).Add(child_widget)
            # We need to call Layout here, because it's not done
            # automatically by wx and otherwise the new item won't
            # be layed out properly.
            self.Layout()
            return res

    def CalcMin(self):
        """ Returns the minimum size for the children this sizer is 
        managing. Since the size of the Dialog is managed externally,
        this always returns (-1, -1).

        """
        component = self._child
        if component is not None:
            res = component.min_size()
        else:
            res = (-1, -1)
        return res
    
    def RecalcSizes(self):
        """ Resizes the child to fit the available space of the scroll
        area.

        """
        component = self._child
        if component is not None:
            component.resize(self.GetSizeTuple())


class WXScrollArea(WXConstraintsWidget, AbstractTkScrollArea):
    """ A Wx implementation of ScrollArea.

    ..Note: The 'always_on' scrollbar policy is not supported on wx. Wx
        *does* allow the scollbars to be always shown, but said window 
        style applies to *both* horizontal and vertical directions, and
        it must be applied at widget creation time. It does not reliably
        toggle on/off dynamically after the widget has been created. As
        it's not reliable, we simply don't support it on wx. Should it
        become a highly desired feature, this decision can be revisited.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QScrollAreacontrol.

        """
        style = wx.HSCROLL | wx.VSCROLL | wx.BORDER_SIMPLE 
        self.widget = wx.ScrolledWindow(parent, style=style)
        self.widget.SetSizer(WXScrollAreaSizer())

    def initialize(self):
        """ Intializes the widget with the attributes of this instance.

        """
        super(WXScrollArea, self).initialize()
        shell = self.shell_obj
        self.set_horizontal_policy(shell.horizontal_scrollbar_policy)
        self.set_vertical_policy(shell.vertical_scrollbar_policy)
        self.update_scrolled_component()

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_horizontal_scrollbar_policy_changed(self, policy):
        """ The change handler for the 'horizontal_scrollbar_policy'
        attribute of the shell object.

        """
        self.set_horizontal_policy(policy)

    def shell_vertical_scrollbar_policy_changed(self, policy):
        """ The change handler for the 'vertical_scrollbar_policy'
        attribute of the shell object.

        """
        self.set_vertical_policy(policy)

    def shell_scrolled_component_changed(self, component):
        """ The change handler for the 'layout_children' attribute of 
        the shell object.

        """
        self.update_scrolled_component()
    
    def vertical_scrollbar_thickness(self):
        """ Returns the pixel thickness of the scrollbar.

        """
        return wx.SystemSettings.GetMetric(wx.SYS_VSCROLL_X)
    
    def horizontal_scrollbar_thickness(self):
        """ Returns the pixel thickness of the scrollbar.

        """
        return wx.SystemSettings.GetMetric(wx.SYS_HSCROLL_Y)

    def scroll_to_position(self, position, margin):
        """ Scrolls the area such that position is visible with a
        minimum of margin points surrounding position.

        """
        widget = self.widget

        pos_x, pos_y = position
        margin_x, margin_y = margin
        scroll_unit_x, scroll_unit_y = widget.GetScrollPixelsPerUnit()
        scroll_unit_x = 1.0 if scroll_unit_x == 0.0 else scroll_unit_x
        scroll_unit_y = 1.0 if scroll_unit_y == 0.0 else scroll_unit_y

        widget.Scroll(max(0.0, (pos_x - margin_x) / float(scroll_unit_x)),
                      max(0.0, (pos_y - margin_y) / float(scroll_unit_y)))

    def size_hint(self):
        """ Returns a (width, height) tuple of integers for the size hint
        of the scroll area. Overridden from the parent class to return
        a reasonable value for a Scroll Area.

        """
        # Trying to have wx compute a decent size hint is probably
        # the most painful thing i've experienced in programming. It's 
        # just not worth the effort. For now, we try to compute from the
        # min size of the sizer, and if that fails just return the size 
        # hint used by QAbstractScrollArea.
        res = self.widget.GetSizer().CalcMin()
        if res == (-1, -1):
            res = (256, 192)
        return res

    #--------------------------------------------------------------------------
    # Widget Update
    #--------------------------------------------------------------------------
    def set_horizontal_policy(self, policy):
        """ Set the horizontal scrollbar policy of the widget.

        """
        horiz = SCROLLBAR_POLICY_MAP[policy]
        vert = SCROLLBAR_POLICY_MAP[self.shell_obj.vertical_scrollbar_policy]
        self.widget.SetScrollRate(horiz, vert)

    def set_vertical_policy(self, policy):
        """ Set the vertical scrollbar policy of the widget.

        """
        horiz = SCROLLBAR_POLICY_MAP[self.shell_obj.horizontal_scrollbar_policy]
        vert = SCROLLBAR_POLICY_MAP[policy]
        self.widget.SetScrollRate(horiz, vert)
   
    def update_scrolled_component(self):
        """ Update the QScrollArea's children with the current children.

        """
        sizer = self.widget.GetSizer()
        sizer.Add(self.shell_obj.scrolled_component)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import Instance

from .toggle_control import ToggleControl, AbstractTkToggleControl


class AbstractTkCheckBox(AbstractTkToggleControl):
    """ The abstract toolkit CheckBox interface.

    """
    pass


class CheckBox(ToggleControl):
    """ A check box widget.

    Use a check box when it's necessary to toggle a boolean value
    independent of any other widgets in the group. For allowing the
    toggling of only one value in a group of values, use a group
    of radio buttons.

    See Also
    --------
    ToggleControl

    """
    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkCheckBox)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import window

class TestQtWindow(QtTestAssistant, window.TestWindow):
    """ QtWindow tests. """

    def get_title(self, widget):
        """ Get a window's title.

        """
        return widget.windowTitle()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import spin_box


class TestQtSpinBox(QtTestAssistant, spin_box.TestSpinBox):
    """ QtSpinBox tests. """

    def get_value(self, widget):
        """ Get a spin box's value.

        """
        return widget.value()

    def get_low(self, widget):
        """ Get a spin box's minimum value.

        """
        return widget.minimum()

    def get_high(self, widget):
        """ Get a spin box's maximum value.

        """
        return widget.maximum()

    def get_step(self, widget):
        """ Get a spin box's step size.

        """
        return widget.singleStep()

    def get_wrap(self, widget):
        """ Check if a spin box wraps around at the edge values.

        """
        return widget.wrapping()

    def get_text(self, widget):
        """ Get the text displayed in a spin box.

        """
        return widget.text()

    def spin_up_event(self, widget):
        """ Simulate a click on the 'up' spin button.

        """
        widget.stepUp()

    def spin_down_event(self, widget):
        """ Simulate a click on the 'down' spin button.

        """
        widget.stepDown()



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" A simple Chaco Plot Component.

"""
import functools

from chaco.api import ArrayPlotData
from chaco.tools.api import PanTool, ZoomTool
from chaco.shell.scaly_plot import ScalyPlot
from chaco.scales.api import CalendarScaleSystem
from traits.api import (
    Array, Bool, DelegatesTo, Dict, Enum, Instance, List, Str, on_trait_change,
)

from enaml.core.toolkit import Toolkit
from enaml.components.enable_canvas import EnableCanvas


class DeferredCall(object):
    def __init__(self, obj, func, *args, **kwds):
        self.obj = obj
        self.func = func
        self.args = args
        self.kwds = kwds

    def __call__(self):
        return self.func(self.obj, *self.args, **self.kwds)


def plot_command(func):
    """ Create an object that represents a call to a method on a Plot 
    object.

    """
    @functools.wraps(func)
    def wrapper(self, *args, **kwds):
        return DeferredCall(self, func, *args, **kwds)
    return wrapper


class SimplePlot(EnableCanvas):
    """ A Component that encapsulates a simple Chaco Plot.

    """

    #: The data.
    data = Dict(Str, Array)

    #: Titles.
    title = Str()
    xtitle = Str()
    ytitle = Str()

    #: The axis scales.
    xscale = Enum('linear', 'time', 'log')
    yscale = Enum('linear', 'time', 'log')

    #: Whether to show the legend or not.
    legend = Bool(False)

    #: The list of commands to execute on the Plot object.
    commands = List(Instance(DeferredCall))

    #: The plot data object.
    plot_data = Instance(ArrayPlotData, ())

    #: The plot object.
    component = Instance(ScalyPlot)
    def _component_default(self):
        plot = ScalyPlot(self.plot_data)
        return plot

    #: Delegates.
    plot_bgcolor = DelegatesTo('component', 'bgcolor')
    default_origin = DelegatesTo('component')
    orientation = DelegatesTo('component')
    padding_top = DelegatesTo('component')
    padding_bottom = DelegatesTo('component')
    padding_left = DelegatesTo('component')
    padding_right = DelegatesTo('component')
    padding = DelegatesTo('component')
    border_visible = DelegatesTo('component')
    title = DelegatesTo('component')
    title_font = DelegatesTo('component')
    title_position = DelegatesTo('component')
    title_text = DelegatesTo('component')
    title_color = DelegatesTo('component')
    title_angle = DelegatesTo('component')
    legend_alignment = DelegatesTo('component')

    @classmethod
    def activate(cls):
        """ Add this component to the active toolkit.

        """
        tk = Toolkit.active_toolkit()
        cons = tk['EnableCanvas'].clone(shell_loader=lambda:cls)
        tk[cls.__name__] = cons

    def _setup_init_widgets(self):
        super(SimplePlot, self)._setup_init_widgets()
        self.set_data()
        self.set_commands()
        self._xtitle_changed(self.xtitle)
        self._ytitle_changed(self.ytitle)
        self._title_changed(self.title)

    @on_trait_change('commands[]')
    def set_commands(self):
        """ Execute the commands on the plot.

        """
        for command in self.commands:
            command()

    @on_trait_change('data[]')
    def set_data(self):
        for name, array in self.data.iteritems():
            self.plot_data.set_data(name, array)

    @plot_command
    def plot(self, *args, **kwds):
        self.component.plot(*args, **kwds)

    @plot_command
    def img_plot(self, *args, **kwds):
        self.component.img_plot(*args, **kwds)

    @plot_command
    def contour_plot(self, *args, **kwds):
        self.component.contour_plot(*args, **kwds)

    @plot_command
    def semilogx(self, *args, **kwds):
        kwds['index_scale'] = 'log'
        self.component.plot(*args, **kwds)

    @plot_command
    def semilogy(self, *args, **kwds):
        kwds['value_scale'] = 'log'
        self.component.plot(*args, **kwds)

    @plot_command
    def loglog(self, *args, **kwds):
        kwds['index_scale'] = 'log'
        kwds['value_scale'] = 'log'
        self.component.plot(*args, **kwds)

    @plot_command
    def xaxis(self, **kwds):
        self.component.x_axis.trait_set(**kwds)
        self.component.request_redraw()

    @plot_command
    def yaxis(self, **kwds):
        self.component.y_axis.trait_set(**kwds)
        self.component.request_redraw()

    @plot_command
    def xrange(self, **kwds):
        self.component.index_range.trait_set(**kwds)

    @plot_command
    def yrange(self, **kwds):
        self.component.value_range.trait_set(**kwds)

    @plot_command
    def pan_tool(self, **kwds):
        self.component.tools.append(PanTool(self.component, **kwds))

    @plot_command
    def zoom_tool(self, **kwds):
        self.component.overlays.append(ZoomTool(self.component, **kwds))

    def _xtitle_changed(self, new):
        self.component.x_axis.title = new
        self.component.request_redraw()

    def _ytitle_changed(self, new):
        self.component.y_axis.title = new
        self.component.request_redraw()

    def _title_changed(self, new):
        self.component.title = new
        self.component.request_redraw()

    def _set_scale(self, axis, system):
        p = self.component
        if axis == 'x':
            log_linear_trait = 'index_scale'
            ticks = p.x_ticks
        else:
            log_linear_trait = 'value_scale'
            ticks = p.y_ticks
        if system == 'time':
            system = CalendarScaleSystem()
        if isinstance(system, basestring):
            setattr(p, log_linear_trait, system)
        else:
            if system is None:
                system = dict(linear=p.linear_scale, log=p.log_scale).get(
                    p.get(log_linear_trait), p.linear_scale)
            ticks.scale = system
        p.request_redraw()

    def _xscale_changed(self, new):
        self._set_scale('x', new)

    def _yscale_changed(self, new):
        self._set_scale('y', new)

    def _legend_changed(self, new):
        self.component.legend = new
        self.component.request_redraw()

    def _plot_bgcolor_changed(self, new):
        self.component.request_redraw()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui, QtCore
from .qt_control import QtControl

from ...components.text_editor import AbstractTkTextEditor
from ...guard import guard


class QtEnamlPlainTextEdit(QtGui.QPlainTextEdit):
    """ A QLineEdit subclass which converts a lost focus event into
    a lost focus signal.

    """
    lostFocus = QtCore.Signal()

    def focusOutEvent(self, event):
        self.lostFocus.emit()
        return super(QtEnamlPlainTextEdit, self).focusOutEvent(event)


class QtTextEditor(QtControl, AbstractTkTextEditor):
    """ A Qt4 implementation of a TextEditor.

    """
    #--------------------------------------------------------------------------
    # SetupMethods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QTextEditor.

        """
        self.widget = QtEnamlPlainTextEdit(parent)

    def initialize(self):
        """ Initializes the attributes of the Qt widget.

        """
        super(QtTextEditor, self).initialize()
        shell = self.shell_obj
        shell._modified = False
        self.set_read_only(shell.read_only)
        self.set_cursor_position(shell.cursor_position)
        self.set_anchor_position(shell.anchor_position)
        self.set_wrap_lines(shell.wrap_lines)
        self.set_overwrite(shell.overwrite)

    def bind(self):
        """ Binds the event handlers for the QLineEdit.

        """
        super(QtTextEditor, self).bind()
        widget = self.widget
        widget.textChanged.connect(self.on_text_updated) # XXX or should we bind to textEdited?
        widget.selectionChanged.connect(self.on_selection)
        widget.cursorPositionChanged.connect(self.on_cursor)
        widget.lostFocus.connect(self.on_lost_focus)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_read_only_changed(self, read_only):
        """ The change handler for the 'read_only' attribute on the
        parent.

        """
        self.set_read_only(read_only)

    def shell_cursor_position_changed(self, cursor_position):
        """ The change handler for the 'cursor_position' attribute on 
        the parent.

        """
        if not guard.guarded(self, 'setting_cursor'):
            self.set_cursor_position(cursor_position)

    def shell_anchor_position_changed(self, anchor_position):
        """ The change handler for the 'anchor_position' attribute on 
        the parent.

        """
        if not guard.guarded(self, 'setting_cursor'):
            self.set_anchor_position(anchor_position)
    
    def shell_overwrite_changed(self, overwrite):
        """ The change handler for the 'overwrite' attribute on the 
        parent.

        """
        self.set_overwrite(overwrite)
    
    def shell_wrap_lines_changed(self, wrap_lines):
        """ The change handler for the 'wrap_lines' attribute on the 
        parent.

        """
        self.set_wrap_lines(wrap_lines)

    def set_selection(self, start, end):
        """ Sets the selection in the widget between the start and 
        end positions, inclusive.

        """
        cursor = self.widget.textCursor()
        cursor.setAnchor(start)
        cursor.setPosition(end)
        self.widget.setTextCursor(cursor)
        self.update_shell_selection()

    def select_all(self):
        """ Select all the text in the line edit.

        If there is no text in the line edit, the selection will be
        empty.

        """
        self.widget.selectAll()
        self.update_shell_selection()

    def deselect(self):
        """ Deselect any selected text.

        Sets a selection with start == stop to deselect the current
        selection. The cursor is placed at the beginning of selection.

        """
        cursor = self.widget.textCursor()
        cursor.clearSelection()
        self.update_shell_selection()

    def clear(self):
        """ Clear the line edit of all text.

        """
        self.widget.clear()
        self.update_shell_selection()

    def backspace(self):
        """ Simple backspace functionality.

        If no text is selected, deletes the character to the left of 
        the cursor. Otherwise, it deletes the selected text.

        """
        cursor = self.widget.textCursor()
        cursor.deletePreviousChar()
        self.update_shell_selection()

    def delete(self):
        """ Simple delete functionality.

        If no text is selected, deletes the character to the right of 
        the cursor. Otherwise, it deletes the selected text.

        """
        cursor = self.widget.textCursor()
        cursor.deleteChar()
        self.update_shell_selection()

    def end(self, mark=False):
        """ Moves the cursor to the end of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            end of the line edit. Defaults to False.

        """
        widget = self.widget
        if mark:
            start = widget.cursorPosition()
            end = len(widget.text())
            widget.setSelection(start, end)
        else:
            cursor = widget.textCursor()
            cursor.movePosition(cursor.End)
        self.update_shell_selection()

    def home(self, mark=False):
        """ Moves the cursor to the beginning of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            beginning of the line edit. Defaults to False.

        """
        widget = self.widget
        if mark:
            start = 0
            end = widget.cursorPosition()
            widget.setSelection(start, end)
        else:
            cursor = widget.textCursor()
            cursor.movePosition(cursor.Start)
        self.update_shell_selection()

    def cut(self):
        """ Cuts the selected text from the line edit.

        Copies the selected text to the clipboard then deletes the 
        selected text from the line edit.

        """
        self.widget.cut()
        self.update_shell_selection()

    def copy(self):
        """ Copies the selected text to the clipboard.

        """
        self.widget.copy()
        self.update_shell_selection()

    def paste(self):
        """ Paste the contents of the clipboard into the line edit.

        Inserts the contents of the clipboard into the line edit at the 
        current cursor position, replacing any selected text.

        """
        self.widget.paste()
        self.update_shell_selection()

    def insert(self, text):
        """ Insert the text into the line edit.

        Inserts the given text at the current cursor position, replacing
        any selected text.

        Arguments
        ---------
        text : str
            The text to insert into the line edit.

        """
        self.widget.insertPlainText(text)
        self.update_shell_selection()

    def undo(self):
        """ Undoes the last operation.

        """
        self.widget.undo()
        self.update_shell_selection()

    def redo(self):
        """ Redoes the last operation

        """
        self.widget.redo()
        self.update_shell_selection()

    def find(self, text, backwards=False, case_sensitive=False, whole_words=False):
        """ Finds the text in the editor and sets the cursor to that 
        position.

        Parameters
        ----------
        text : string
            The text to find in the buffer

        backwards : bool, optional
            If True search starting at the end of the buffer. Defaults
            to False.
        
        case_sensitive : bool, optional
            If True, the text matching is case sensitive. Defaults to 
            False.
        
        whole_words : bool, optional
            If True, only complete words are matched. Defaults to False.

        Returns
        -------
        result : bool
            True if the text was found, False otherwise.
        
        """
        options = QtGui.QTextDocument.FindFlags()
        if backwards:
            options |= QtGui.QTextDocument.FindBackward
        if case_sensitive:
            options |= QtGui.QTextDocument.FindCaseSensitively
        if whole_words:
            options |= QtGui.QTextDocument.FindWholeWords
        return self.widget.find(text, options)

    def on_text_updated(self):
        """ The event handler for the text update event.

        """
        shell = self.shell_obj
        self.update_shell_selection()
        shell._modified = True
        shell.text_edited()
        shell.text_changed()

    def on_selection(self):
        """ The event handler for a selection (really a left up) event.

        """
        self.update_shell_selection()

    def on_cursor(self):
        """ The event handler for a cursor move.

        """
        self.update_shell_selection()

    def on_lost_focus(self):
        """ The event handler for focus lost event.

        """
        self.shell_obj.lost_focus()

    def update_shell_selection(self):
        """ Updates the selection and cursor position of the parent to 
        reflect the current state of the widget.

        """
        shell = self.shell_obj
        cursor = self.widget.textCursor()
        selected_text = cursor.selectedText()
        selected_text = selected_text.replace(u'\u2029', '\n') # replace unicode line break
        shell._selected_text = selected_text
        with guard(self, 'setting_cursor'):
            shell.cursor_position = cursor.position()
            shell.anchor_position = cursor.anchor()
            shell._cursor_column = cursor.positionInBlock()
            shell._cursor_line = cursor.blockNumber()

    def get_text(self):
        """ Get the text currently in the widget.

        """
        text = self.widget.toPlainText()
        return text

    def set_text(self, text):
        """ Changes the text in the widget without emitted a text 
        updated event. This should be called when the text is changed
        programmatically.

        """
        self.widget.setPlainText(text)
    
    def set_read_only(self, read_only):
        """ Sets read only state of the widget.

        """
        self.widget.setReadOnly(read_only)

    def set_cursor_position(self, cursor_position):
        """ Sets the cursor position of the widget.

        """
        cursor = self.widget.textCursor()
        cursor.setPosition(cursor_position)

    def set_anchor_position(self, anchor_position):
        """ Sets the cursor position of the widget.

        """
        cursor = self.widget.textCursor()
        cursor.setPosition(anchor_position)

    def set_wrap_lines(self, wrap_lines):
        """ Sets the line wrapping of the widget.

        """
        if wrap_lines:
            self.widget.setLineWrapMode(QtGui.QPlainTextEdit.WidgetWidth)
        else:
            self.widget.setLineWrapMode(QtGui.QPlainTextEdit.NoWrap)

    def set_overwrite(self, overwrite):
        """ Sets the overwrite mode of the widget.

        """
        self.widget.setOverwriteMode(overwrite)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestLessLess(EnamlTestCase):
    """ Generic testcases for enaml the LessLess operator.

    These testcases test the basic functionality of widgets interacting
    with each other.

    To run these tests for a specific back-end it is required to have
    a functional implementation of the following wigdets:
        - Window
        - CheckBox
        - Label

    Abstract Methods
    ----------------
    get_text(widget)
        Returns the text from the tookit widget of a Label.

    get_checked(widget)
        Return the checked status of a CheckBox.

    """
    def setUp(self):
        enaml_source = """
enamldef MainView(MainWindow):
    Container:
        Label:
            name = 'lb'
            text << 'CheckBox is {0}'.format(cb.checked)
        CheckBox:
            id: cb
            name = 'cb'
            text = 'Should be checked'
            checked = True
"""
        self.view = self.parse_and_create(enaml_source)
        self.label = self.component_by_name(self.view, 'lb')
        self.check_box = self.component_by_name(self.view, 'cb')
        self.label_widget = self.label.toolkit_widget
        self.check_box_widget = self.check_box.toolkit_widget

    def test_lessless_init(self):
        """ Test the value after initialization

        """
        label = self.label
        check_box = self.check_box
        self.assertEnamlInSync(check_box, 'checked', True)
        self.assertEnamlInSync(label, 'text', 'CheckBox is True')

    def test_lessless_update(self):
        """ Test the value after initialization

        """
        label = self.label
        check_box = self.check_box
        check_box.checked = False
        self.assertEnamlInSync(label, 'text', 'CheckBox is False')
        self.assertEnamlInSync(check_box, 'checked', False)

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_text(self, widget):
        """ Returns the label text from the tookit widget of Label.

        """
        pass

    @required_method
    def get_checked(self, widget):
        """ Returns the label text from the tookit widget of CheckBox.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import os

import wx

from .wx_dialog import WXDialog

from ...components.directory_dialog import AbstractTkDirectoryDialog
from ...guard import guard


class WXDirectoryDialog(WXDialog, AbstractTkDirectoryDialog):
    """ A Wx implementation of a DirectoryDialog.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.DirDialog control.

        """
        self.widget = wx.DirDialog(parent)

    def initialize(self):
        """ Initializes the attributes of the underlying wx.DirDialog.

        """
        super(WXDirectoryDialog, self).initialize()
        shell = self.shell_obj
        self.set_directory(shell.directory)
    
    # The wx.DirDialog doesn't emit events for anything useful, 
    # so there is nothing which need be bound.

    #--------------------------------------------------------------------------
    # Shell Object Change Handlers
    #--------------------------------------------------------------------------
    def shell_directory_changed(self, directory):
        """ Update the dialog with the new working directory.

        """
        if not guard.guarded(self, 'setting_directory'):
            self.set_directory(directory)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_directory(self, directory):
        """ Sets the current directory of the dialog.

        """
        self.widget.SetPath(directory)

    def set_central_widget(self, widget):
        """ Overridden parent class method which makes central widget
        operations a no-op, since a directory dialog has no children.

        """
        pass

    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def _handle_retcode(self, retcode):
        """ Overridden parent class method to update the shell object
        with new info once the dialog is closed.

        """
        widget = self.widget
        shell = self.shell_obj
        if retcode == wx.ID_OK:
            path = widget.GetPath()
            with guard(self, 'setting_directory'):
                shell.directory = path
        super(WXDirectoryDialog, self)._handle_retcode(retcode)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_test_assistant import WXTestAssistant, skip_nonwindows

from .. import toggle_button

@skip_nonwindows
class TestWXToggleButton(WXTestAssistant, toggle_button.TestToggleButton):
    """ WXToggleButton tests. 
    
    """
    def get_text(self, widget):
        """ Returns the text from the tookit widget.

        """
        return widget.GetLabel()

    def get_checked(self, widget):
        """ Returns the checked status from the tookit widget.

        """
        return widget.GetValue()

    def toggle_button_pressed(self, widget):
        """ Press the toggle button programmatically.

        """
        self.send_wx_event(widget, wx.EVT_LEFT_DOWN)

    def toggle_button_released(self, widget):
        """ Release the button programmatically.

        """
        self.send_wx_event(widget, wx.EVT_LEFT_UP)
        self.send_wx_event(widget, wx.EVT_LEAVE_WINDOW)

    def toggle_button_toggle(self, widget):
        """ Toggle the button programmatically.

        """
        self.send_wx_event(widget, wx.EVT_TOGGLEBUTTON)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import unittest
from datetime import date

from traits.api import (HasStrictTraits, TraitError, Float, Instance, Date)

from enaml.core.trait_types import Bounded

class Test_Bounded_Static(unittest.TestCase):
    """ Test the use of the Bounded trait with static bounds.

    """
    @classmethod
    def setUpClass(self):

        class my_class(HasStrictTraits):

            value = Bounded(0.2, 0, 4)

        self.traits_class = my_class

    def setUp(self):
        self.traits_instance = self.traits_class()

    def test_init(self):
        """ Test basic static initialization """
        value = self.traits_instance.value
        self.assertAlmostEqual(value, 0.2)

    def test_assigment(self):
        """ Test static assigment """
        instance = self.traits_instance
        instance.value = 0.7
        self.assertAlmostEqual(instance.value, 0.7)

    def test_invalid(self):
        """ Test static assigment """
        instance = self.traits_instance
        with self.assertRaises(TraitError):
            instance.value = -2

class Test_Bounded_Dynamic(unittest.TestCase):
    """ Test the use of the Bounded trait with dynamic bounds.

    """
    @classmethod
    def setUpClass(self):

        class my_class(HasStrictTraits):

            low = Float(0)
            high = Float(4)
            value = Bounded(0.2, low='low', high='high')

        self.traits_class = my_class

    def setUp(self):
        self.traits_instance = self.traits_class()

    def test_init(self):
        """ Test dynamic initialization. """
        value = self.traits_instance.value
        self.assertAlmostEqual(value, 0.2)

    def test_assigment(self):
        """ Test assigment. """
        instance = self.traits_instance
        instance.value = 0.7
        self.assertAlmostEqual(instance.value, 0.7)

    def test_invalid(self):
        """ Test invalid assigment. """
        instance = self.traits_instance
        with self.assertRaises(TraitError):
            instance.value = -2

    def test_change_lower(self):
        """ Test changing the lower bound. """
        instance = self.traits_instance
        instance.low = -4.0
        instance.value = -2
        self.assertAlmostEqual(instance.value, -2)

    def test_change_upper(self):
        """ Test changing the upper bound. """
        instance = self.traits_instance
        instance.high =  6.0
        instance.value =  5.7
        self.assertAlmostEqual(instance.value, 5.7)

class Test_Bounded_Special(unittest.TestCase):
    """ Test special use cases for the Bounded trait.

    """
    def test_inner_bound_class(self):
        """ Test dynamic initialization with inner class.

        """
        class small_class(HasStrictTraits):
            low = Float(0)
            high = Float(2)

        class main_class(HasStrictTraits):
            bounds = Instance(small_class, ())
            value = Bounded(0.2, 'bounds.low', 'bounds.high')

        instance = main_class()
        instance.value = 0.2
        self.assertAlmostEqual(instance.value, 0.2)

        with self.assertRaises(TraitError):
            instance.value = -1

    def test_bounded_traits(self):
        """ Test initialization with Trait Class

        """
        class main_class(HasStrictTraits):
            value = Bounded(Date(date(2007,12, 18)),
                            date(2003,12, 18),
                            date(2010,12, 18))

        instance = main_class()
        self.assertEqual(instance.value, date(2007,12, 18))

        with self.assertRaises(TraitError):
            instance.value = 0.2

        instance.value = date(2008,12, 18)
        self.assertEqual(instance.value, date(2008,12, 18))

    def test_bounded_python(self):
        """ Test initialization wiht complex python object.

        """
        class main_class(HasStrictTraits):
            value = Bounded(date(2007,12, 18),
                            date(2003,12, 18),
                            date(2010,12, 18))

        instance = main_class()
        self.assertEqual(instance.value, date(2007,12, 18))
        with self.assertRaises(TraitError):
            instance.value = 0.2

        instance.value = date(2008,12, 18)
        self.assertEqual(instance.value, date(2008,12, 18))


#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" A fake data generated which can generate data at various frequencies.
This is not intended for production use, but rather as a stand-in for
some external data generation feed.

"""
from abc import ABCMeta, abstractmethod
from collections import defaultdict
import threading
import time


class AbstractDataGenerator(object):
    """ An abstract base class which defines the dummy api for a 
    data generator.

    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def subscribe(self, id, callback):
        """ Register a callback to be called when data is available
        for the given id.
        
        """
        raise NotImplementedError
        
 
class DummyDataGenerator(object):
    """ A dummy data generator which generates timestamp data on a
    specified frequency.

    This class is for demonstration purposes only, and will be 
    replaced with an actualy production data source.

    """
    def __init__(self, frequency, *value_gens):
        """ Initialize a DummyDataGenerator

        Parameters
        ----------
        frequencey : float
            A floating point frequency (Hz) which indicates how
            freqeuntly data should be generated.
        
        *value_gens : callables
            Callables which are called with the time stamp and return
            a tuple of (id, value) where id is a string representing
            the feed id and value is the datapoint for the time stamp.
            These values will be republished to any listeners.

        """
        self.callback = None
        self.go = True
        self.frequency = frequency
        self.callbacks = defaultdict(list)
        self.value_gens = value_gens

    def start(self):
        """ Start the generator's generation thread. Any registered
        callbacks will be called when new data is available.

        """
        def inner(data_generator):
            while(data_generator.go):
                time_stamp =  time.clock()
                callbacks = data_generator.callbacks
                for gen in data_generator.value_gens:
                    id, data = gen(time_stamp)
                    for cb in callbacks[id]:
                        cb((id, time_stamp, data))
                time.sleep(1.0 / data_generator.frequency)
        thread = threading.Thread(target=inner, args=(self,))
        thread.daemon = True
        thread.start()

    def stop(self):
        """ Stop the generator's generation thread.

        """
        self.go = False

    def subscribe(self, id, callback):
        """ Register a callback to be called when data is available
        for the given id.
        
        """
        self.callbacks[id].append(callback)

